# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: injective/exchange/v1beta1/events.proto
"""Generated protocol buffer code."""
from google.protobuf.internal import builder as _builder
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from gogoproto import gogo_pb2 as gogoproto_dot_gogo__pb2
from cosmos.base.v1beta1 import coin_pb2 as cosmos_dot_base_dot_v1beta1_dot_coin__pb2
from injective.oracle.v1beta1 import oracle_pb2 as injective_dot_oracle_dot_v1beta1_dot_oracle__pb2
from injective.exchange.v1beta1 import exchange_pb2 as injective_dot_exchange_dot_v1beta1_dot_exchange__pb2


DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n\'injective/exchange/v1beta1/events.proto\x12\x1ainjective.exchange.v1beta1\x1a\x14gogoproto/gogo.proto\x1a\x1e\x63osmos/base/v1beta1/coin.proto\x1a%injective/oracle/v1beta1/oracle.proto\x1a)injective/exchange/v1beta1/exchange.proto\"\xb4\x01\n\x17\x45ventBatchSpotExecution\x12\x11\n\tmarket_id\x18\x01 \x01(\t\x12\x0e\n\x06is_buy\x18\x02 \x01(\x08\x12@\n\rexecutionType\x18\x03 \x01(\x0e\x32).injective.exchange.v1beta1.ExecutionType\x12\x34\n\x06trades\x18\x04 \x03(\x0b\x32$.injective.exchange.v1beta1.TradeLog\"\xa8\x02\n\x1d\x45ventBatchDerivativeExecution\x12\x11\n\tmarket_id\x18\x01 \x01(\t\x12\x0e\n\x06is_buy\x18\x02 \x01(\x08\x12\x16\n\x0eis_liquidation\x18\x03 \x01(\x08\x12J\n\x12\x63umulative_funding\x18\x04 \x01(\tB.\xda\xde\x1f&github.com/cosmos/cosmos-sdk/types.Dec\xc8\xde\x1f\x01\x12@\n\rexecutionType\x18\x05 \x01(\x0e\x32).injective.exchange.v1beta1.ExecutionType\x12>\n\x06trades\x18\x06 \x03(\x0b\x32..injective.exchange.v1beta1.DerivativeTradeLog\"\x81\x02\n\x1d\x45ventLostFundsFromLiquidation\x12\x11\n\tmarket_id\x18\x01 \x01(\t\x12\x15\n\rsubaccount_id\x18\x02 \x01(\x0c\x12_\n\'lost_funds_from_available_during_payout\x18\x03 \x01(\tB.\xda\xde\x1f&github.com/cosmos/cosmos-sdk/types.Dec\xc8\xde\x1f\x00\x12U\n\x1dlost_funds_from_order_cancels\x18\x04 \x01(\tB.\xda\xde\x1f&github.com/cosmos/cosmos-sdk/types.Dec\xc8\xde\x1f\x00\"t\n\x1c\x45ventBatchDerivativePosition\x12\x11\n\tmarket_id\x18\x01 \x01(\t\x12\x41\n\tpositions\x18\x02 \x03(\x0b\x32..injective.exchange.v1beta1.SubaccountPosition\"\x7f\n\x1b\x45ventDerivativeMarketPaused\x12\x11\n\tmarket_id\x18\x01 \x01(\t\x12\x14\n\x0csettle_price\x18\x02 \x01(\t\x12\x1b\n\x13total_missing_funds\x18\x03 \x01(\t\x12\x1a\n\x12missing_funds_rate\x18\x04 \x01(\t\"d\n\x1b\x45ventMarketBeyondBankruptcy\x12\x11\n\tmarket_id\x18\x01 \x01(\t\x12\x14\n\x0csettle_price\x18\x02 \x01(\t\x12\x1c\n\x14missing_market_funds\x18\x03 \x01(\t\"_\n\x18\x45ventAllPositionsHaircut\x12\x11\n\tmarket_id\x18\x01 \x01(\t\x12\x14\n\x0csettle_price\x18\x02 \x01(\t\x12\x1a\n\x12missing_funds_rate\x18\x03 \x01(\t\"g\n\x1e\x45ventBinaryOptionsMarketUpdate\x12\x45\n\x06market\x18\x01 \x01(\x0b\x32/.injective.exchange.v1beta1.BinaryOptionsMarketB\x04\xc8\xde\x1f\x00\"\xa8\x01\n\x12\x45ventNewSpotOrders\x12\x11\n\tmarket_id\x18\x01 \x01(\t\x12>\n\nbuy_orders\x18\x02 \x03(\x0b\x32*.injective.exchange.v1beta1.SpotLimitOrder\x12?\n\x0bsell_orders\x18\x03 \x03(\x0b\x32*.injective.exchange.v1beta1.SpotLimitOrder\"\xba\x01\n\x18\x45ventNewDerivativeOrders\x12\x11\n\tmarket_id\x18\x01 \x01(\t\x12\x44\n\nbuy_orders\x18\x02 \x03(\x0b\x32\x30.injective.exchange.v1beta1.DerivativeLimitOrder\x12\x45\n\x0bsell_orders\x18\x03 \x03(\x0b\x32\x30.injective.exchange.v1beta1.DerivativeLimitOrder\"j\n\x14\x45ventCancelSpotOrder\x12\x11\n\tmarket_id\x18\x01 \x01(\t\x12?\n\x05order\x18\x02 \x01(\x0b\x32*.injective.exchange.v1beta1.SpotLimitOrderB\x04\xc8\xde\x1f\x00\"U\n\x15\x45ventSpotMarketUpdate\x12<\n\x06market\x18\x01 \x01(\x0b\x32&.injective.exchange.v1beta1.SpotMarketB\x04\xc8\xde\x1f\x00\"\x81\x02\n\x1a\x45ventPerpetualMarketUpdate\x12\x42\n\x06market\x18\x01 \x01(\x0b\x32,.injective.exchange.v1beta1.DerivativeMarketB\x04\xc8\xde\x1f\x00\x12T\n\x15perpetual_market_info\x18\x02 \x01(\x0b\x32/.injective.exchange.v1beta1.PerpetualMarketInfoB\x04\xc8\xde\x1f\x01\x12I\n\x07\x66unding\x18\x03 \x01(\x0b\x32\x32.injective.exchange.v1beta1.PerpetualMarketFundingB\x04\xc8\xde\x1f\x01\"\xc3\x01\n\x1e\x45ventExpiryFuturesMarketUpdate\x12\x42\n\x06market\x18\x01 \x01(\x0b\x32,.injective.exchange.v1beta1.DerivativeMarketB\x04\xc8\xde\x1f\x00\x12]\n\x1a\x65xpiry_futures_market_info\x18\x03 \x01(\x0b\x32\x33.injective.exchange.v1beta1.ExpiryFuturesMarketInfoB\x04\xc8\xde\x1f\x01\"\xa6\x02\n!EventPerpetualMarketFundingUpdate\x12\x11\n\tmarket_id\x18\x01 \x01(\t\x12I\n\x07\x66unding\x18\x02 \x01(\x0b\x32\x32.injective.exchange.v1beta1.PerpetualMarketFundingB\x04\xc8\xde\x1f\x00\x12\x19\n\x11is_hourly_funding\x18\x03 \x01(\x08\x12\x44\n\x0c\x66unding_rate\x18\x04 \x01(\tB.\xda\xde\x1f&github.com/cosmos/cosmos-sdk/types.Dec\xc8\xde\x1f\x01\x12\x42\n\nmark_price\x18\x05 \x01(\tB.\xda\xde\x1f&github.com/cosmos/cosmos-sdk/types.Dec\xc8\xde\x1f\x01\"u\n\x16\x45ventSubaccountDeposit\x12\x13\n\x0bsrc_address\x18\x01 \x01(\t\x12\x15\n\rsubaccount_id\x18\x02 \x01(\x0c\x12/\n\x06\x61mount\x18\x03 \x01(\x0b\x32\x19.cosmos.base.v1beta1.CoinB\x04\xc8\xde\x1f\x00\"v\n\x17\x45ventSubaccountWithdraw\x12\x15\n\rsubaccount_id\x18\x01 \x01(\x0c\x12\x13\n\x0b\x64st_address\x18\x02 \x01(\t\x12/\n\x06\x61mount\x18\x03 \x01(\x0b\x32\x19.cosmos.base.v1beta1.CoinB\x04\xc8\xde\x1f\x00\"\x87\x01\n\x1e\x45ventSubaccountBalanceTransfer\x12\x19\n\x11src_subaccount_id\x18\x01 \x01(\t\x12\x19\n\x11\x64st_subaccount_id\x18\x02 \x01(\t\x12/\n\x06\x61mount\x18\x03 \x01(\x0b\x32\x19.cosmos.base.v1beta1.CoinB\x04\xc8\xde\x1f\x00\"]\n\x17\x45ventBatchDepositUpdate\x12\x42\n\x0f\x64\x65posit_updates\x18\x01 \x03(\x0b\x32).injective.exchange.v1beta1.DepositUpdate\"\xb5\x01\n\x1b\x44\x65rivativeMarketOrderCancel\x12M\n\x0cmarket_order\x18\x01 \x01(\x0b\x32\x31.injective.exchange.v1beta1.DerivativeMarketOrderB\x04\xc8\xde\x1f\x01\x12G\n\x0f\x63\x61ncel_quantity\x18\x02 \x01(\tB.\xda\xde\x1f&github.com/cosmos/cosmos-sdk/types.Dec\xc8\xde\x1f\x00\"\xef\x01\n\x1a\x45ventCancelDerivativeOrder\x12\x11\n\tmarket_id\x18\x01 \x01(\t\x12\x15\n\risLimitCancel\x18\x02 \x01(\x08\x12K\n\x0blimit_order\x18\x03 \x01(\x0b\x32\x30.injective.exchange.v1beta1.DerivativeLimitOrderB\x04\xc8\xde\x1f\x01\x12Z\n\x13market_order_cancel\x18\x04 \x01(\x0b\x32\x37.injective.exchange.v1beta1.DerivativeMarketOrderCancelB\x04\xc8\xde\x1f\x01\"]\n\x18\x45ventFeeDiscountSchedule\x12\x41\n\x08schedule\x18\x01 \x01(\x0b\x32/.injective.exchange.v1beta1.FeeDiscountSchedule\"\xbf\x01\n EventTradingRewardCampaignUpdate\x12L\n\rcampaign_info\x18\x01 \x01(\x0b\x32\x35.injective.exchange.v1beta1.TradingRewardCampaignInfo\x12M\n\x15\x63\x61mpaign_reward_pools\x18\x02 \x03(\x0b\x32..injective.exchange.v1beta1.CampaignRewardPool\"e\n\x1e\x45ventTradingRewardDistribution\x12\x43\n\x0f\x61\x63\x63ount_rewards\x18\x01 \x03(\x0b\x32*.injective.exchange.v1beta1.AccountRewards\"\x94\x01\n\"EventNewConditionalDerivativeOrder\x12\x11\n\tmarket_id\x18\x01 \x01(\t\x12:\n\x05order\x18\x02 \x01(\x0b\x32+.injective.exchange.v1beta1.DerivativeOrder\x12\x0c\n\x04hash\x18\x03 \x01(\x0c\x12\x11\n\tis_market\x18\x04 \x01(\x08\"\xed\x01\n%EventCancelConditionalDerivativeOrder\x12\x11\n\tmarket_id\x18\x01 \x01(\t\x12\x15\n\risLimitCancel\x18\x02 \x01(\x08\x12K\n\x0blimit_order\x18\x03 \x01(\x0b\x32\x30.injective.exchange.v1beta1.DerivativeLimitOrderB\x04\xc8\xde\x1f\x01\x12M\n\x0cmarket_order\x18\x04 \x01(\x0b\x32\x31.injective.exchange.v1beta1.DerivativeMarketOrderB\x04\xc8\xde\x1f\x01\"\x8c\x01\n&EventConditionalDerivativeOrderTrigger\x12\x11\n\tmarket_id\x18\x01 \x01(\x0c\x12\x16\n\x0eisLimitTrigger\x18\x02 \x01(\x08\x12\x1c\n\x14triggered_order_hash\x18\x03 \x01(\x0c\x12\x19\n\x11placed_order_hash\x18\x04 \x01(\x0c\"@\n\x0e\x45ventOrderFail\x12\x0f\n\x07\x61\x63\x63ount\x18\x01 \x01(\x0c\x12\x0e\n\x06hashes\x18\x02 \x03(\x0c\x12\r\n\x05\x66lags\x18\x03 \x03(\r\"~\n+EventAtomicMarketOrderFeeMultipliersUpdated\x12O\n\x16market_fee_multipliers\x18\x01 \x03(\x0b\x32/.injective.exchange.v1beta1.MarketFeeMultiplier\"\xa2\x01\n\x14\x45ventOrderbookUpdate\x12\x41\n\x0cspot_updates\x18\x01 \x03(\x0b\x32+.injective.exchange.v1beta1.OrderbookUpdate\x12G\n\x12\x64\x65rivative_updates\x18\x02 \x03(\x0b\x32+.injective.exchange.v1beta1.OrderbookUpdate\"X\n\x0fOrderbookUpdate\x12\x0b\n\x03seq\x18\x01 \x01(\x04\x12\x38\n\torderbook\x18\x02 \x01(\x0b\x32%.injective.exchange.v1beta1.Orderbook\"\x8d\x01\n\tOrderbook\x12\x11\n\tmarket_id\x18\x01 \x01(\x0c\x12\x35\n\nbuy_levels\x18\x02 \x03(\x0b\x32!.injective.exchange.v1beta1.Level\x12\x36\n\x0bsell_levels\x18\x03 \x03(\x0b\x32!.injective.exchange.v1beta1.LevelBPZNgithub.com/InjectiveLabs/injective-core/injective-chain/modules/exchange/typesb\x06proto3')

_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, globals())
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'injective.exchange.v1beta1.events_pb2', globals())
if _descriptor._USE_C_DESCRIPTORS == False:

  DESCRIPTOR._options = None
  DESCRIPTOR._serialized_options = b'ZNgithub.com/InjectiveLabs/injective-core/injective-chain/modules/exchange/types'
  _EVENTBATCHDERIVATIVEEXECUTION.fields_by_name['cumulative_funding']._options = None
  _EVENTBATCHDERIVATIVEEXECUTION.fields_by_name['cumulative_funding']._serialized_options = b'\332\336\037&github.com/cosmos/cosmos-sdk/types.Dec\310\336\037\001'
  _EVENTLOSTFUNDSFROMLIQUIDATION.fields_by_name['lost_funds_from_available_during_payout']._options = None
  _EVENTLOSTFUNDSFROMLIQUIDATION.fields_by_name['lost_funds_from_available_during_payout']._serialized_options = b'\332\336\037&github.com/cosmos/cosmos-sdk/types.Dec\310\336\037\000'
  _EVENTLOSTFUNDSFROMLIQUIDATION.fields_by_name['lost_funds_from_order_cancels']._options = None
  _EVENTLOSTFUNDSFROMLIQUIDATION.fields_by_name['lost_funds_from_order_cancels']._serialized_options = b'\332\336\037&github.com/cosmos/cosmos-sdk/types.Dec\310\336\037\000'
  _EVENTBINARYOPTIONSMARKETUPDATE.fields_by_name['market']._options = None
  _EVENTBINARYOPTIONSMARKETUPDATE.fields_by_name['market']._serialized_options = b'\310\336\037\000'
  _EVENTCANCELSPOTORDER.fields_by_name['order']._options = None
  _EVENTCANCELSPOTORDER.fields_by_name['order']._serialized_options = b'\310\336\037\000'
  _EVENTSPOTMARKETUPDATE.fields_by_name['market']._options = None
  _EVENTSPOTMARKETUPDATE.fields_by_name['market']._serialized_options = b'\310\336\037\000'
  _EVENTPERPETUALMARKETUPDATE.fields_by_name['market']._options = None
  _EVENTPERPETUALMARKETUPDATE.fields_by_name['market']._serialized_options = b'\310\336\037\000'
  _EVENTPERPETUALMARKETUPDATE.fields_by_name['perpetual_market_info']._options = None
  _EVENTPERPETUALMARKETUPDATE.fields_by_name['perpetual_market_info']._serialized_options = b'\310\336\037\001'
  _EVENTPERPETUALMARKETUPDATE.fields_by_name['funding']._options = None
  _EVENTPERPETUALMARKETUPDATE.fields_by_name['funding']._serialized_options = b'\310\336\037\001'
  _EVENTEXPIRYFUTURESMARKETUPDATE.fields_by_name['market']._options = None
  _EVENTEXPIRYFUTURESMARKETUPDATE.fields_by_name['market']._serialized_options = b'\310\336\037\000'
  _EVENTEXPIRYFUTURESMARKETUPDATE.fields_by_name['expiry_futures_market_info']._options = None
  _EVENTEXPIRYFUTURESMARKETUPDATE.fields_by_name['expiry_futures_market_info']._serialized_options = b'\310\336\037\001'
  _EVENTPERPETUALMARKETFUNDINGUPDATE.fields_by_name['funding']._options = None
  _EVENTPERPETUALMARKETFUNDINGUPDATE.fields_by_name['funding']._serialized_options = b'\310\336\037\000'
  _EVENTPERPETUALMARKETFUNDINGUPDATE.fields_by_name['funding_rate']._options = None
  _EVENTPERPETUALMARKETFUNDINGUPDATE.fields_by_name['funding_rate']._serialized_options = b'\332\336\037&github.com/cosmos/cosmos-sdk/types.Dec\310\336\037\001'
  _EVENTPERPETUALMARKETFUNDINGUPDATE.fields_by_name['mark_price']._options = None
  _EVENTPERPETUALMARKETFUNDINGUPDATE.fields_by_name['mark_price']._serialized_options = b'\332\336\037&github.com/cosmos/cosmos-sdk/types.Dec\310\336\037\001'
  _EVENTSUBACCOUNTDEPOSIT.fields_by_name['amount']._options = None
  _EVENTSUBACCOUNTDEPOSIT.fields_by_name['amount']._serialized_options = b'\310\336\037\000'
  _EVENTSUBACCOUNTWITHDRAW.fields_by_name['amount']._options = None
  _EVENTSUBACCOUNTWITHDRAW.fields_by_name['amount']._serialized_options = b'\310\336\037\000'
  _EVENTSUBACCOUNTBALANCETRANSFER.fields_by_name['amount']._options = None
  _EVENTSUBACCOUNTBALANCETRANSFER.fields_by_name['amount']._serialized_options = b'\310\336\037\000'
  _DERIVATIVEMARKETORDERCANCEL.fields_by_name['market_order']._options = None
  _DERIVATIVEMARKETORDERCANCEL.fields_by_name['market_order']._serialized_options = b'\310\336\037\001'
  _DERIVATIVEMARKETORDERCANCEL.fields_by_name['cancel_quantity']._options = None
  _DERIVATIVEMARKETORDERCANCEL.fields_by_name['cancel_quantity']._serialized_options = b'\332\336\037&github.com/cosmos/cosmos-sdk/types.Dec\310\336\037\000'
  _EVENTCANCELDERIVATIVEORDER.fields_by_name['limit_order']._options = None
  _EVENTCANCELDERIVATIVEORDER.fields_by_name['limit_order']._serialized_options = b'\310\336\037\001'
  _EVENTCANCELDERIVATIVEORDER.fields_by_name['market_order_cancel']._options = None
  _EVENTCANCELDERIVATIVEORDER.fields_by_name['market_order_cancel']._serialized_options = b'\310\336\037\001'
  _EVENTCANCELCONDITIONALDERIVATIVEORDER.fields_by_name['limit_order']._options = None
  _EVENTCANCELCONDITIONALDERIVATIVEORDER.fields_by_name['limit_order']._serialized_options = b'\310\336\037\001'
  _EVENTCANCELCONDITIONALDERIVATIVEORDER.fields_by_name['market_order']._options = None
  _EVENTCANCELCONDITIONALDERIVATIVEORDER.fields_by_name['market_order']._serialized_options = b'\310\336\037\001'
  _EVENTBATCHSPOTEXECUTION._serialized_start=208
  _EVENTBATCHSPOTEXECUTION._serialized_end=388
  _EVENTBATCHDERIVATIVEEXECUTION._serialized_start=391
  _EVENTBATCHDERIVATIVEEXECUTION._serialized_end=687
  _EVENTLOSTFUNDSFROMLIQUIDATION._serialized_start=690
  _EVENTLOSTFUNDSFROMLIQUIDATION._serialized_end=947
  _EVENTBATCHDERIVATIVEPOSITION._serialized_start=949
  _EVENTBATCHDERIVATIVEPOSITION._serialized_end=1065
  _EVENTDERIVATIVEMARKETPAUSED._serialized_start=1067
  _EVENTDERIVATIVEMARKETPAUSED._serialized_end=1194
  _EVENTMARKETBEYONDBANKRUPTCY._serialized_start=1196
  _EVENTMARKETBEYONDBANKRUPTCY._serialized_end=1296
  _EVENTALLPOSITIONSHAIRCUT._serialized_start=1298
  _EVENTALLPOSITIONSHAIRCUT._serialized_end=1393
  _EVENTBINARYOPTIONSMARKETUPDATE._serialized_start=1395
  _EVENTBINARYOPTIONSMARKETUPDATE._serialized_end=1498
  _EVENTNEWSPOTORDERS._serialized_start=1501
  _EVENTNEWSPOTORDERS._serialized_end=1669
  _EVENTNEWDERIVATIVEORDERS._serialized_start=1672
  _EVENTNEWDERIVATIVEORDERS._serialized_end=1858
  _EVENTCANCELSPOTORDER._serialized_start=1860
  _EVENTCANCELSPOTORDER._serialized_end=1966
  _EVENTSPOTMARKETUPDATE._serialized_start=1968
  _EVENTSPOTMARKETUPDATE._serialized_end=2053
  _EVENTPERPETUALMARKETUPDATE._serialized_start=2056
  _EVENTPERPETUALMARKETUPDATE._serialized_end=2313
  _EVENTEXPIRYFUTURESMARKETUPDATE._serialized_start=2316
  _EVENTEXPIRYFUTURESMARKETUPDATE._serialized_end=2511
  _EVENTPERPETUALMARKETFUNDINGUPDATE._serialized_start=2514
  _EVENTPERPETUALMARKETFUNDINGUPDATE._serialized_end=2808
  _EVENTSUBACCOUNTDEPOSIT._serialized_start=2810
  _EVENTSUBACCOUNTDEPOSIT._serialized_end=2927
  _EVENTSUBACCOUNTWITHDRAW._serialized_start=2929
  _EVENTSUBACCOUNTWITHDRAW._serialized_end=3047
  _EVENTSUBACCOUNTBALANCETRANSFER._serialized_start=3050
  _EVENTSUBACCOUNTBALANCETRANSFER._serialized_end=3185
  _EVENTBATCHDEPOSITUPDATE._serialized_start=3187
  _EVENTBATCHDEPOSITUPDATE._serialized_end=3280
  _DERIVATIVEMARKETORDERCANCEL._serialized_start=3283
  _DERIVATIVEMARKETORDERCANCEL._serialized_end=3464
  _EVENTCANCELDERIVATIVEORDER._serialized_start=3467
  _EVENTCANCELDERIVATIVEORDER._serialized_end=3706
  _EVENTFEEDISCOUNTSCHEDULE._serialized_start=3708
  _EVENTFEEDISCOUNTSCHEDULE._serialized_end=3801
  _EVENTTRADINGREWARDCAMPAIGNUPDATE._serialized_start=3804
  _EVENTTRADINGREWARDCAMPAIGNUPDATE._serialized_end=3995
  _EVENTTRADINGREWARDDISTRIBUTION._serialized_start=3997
  _EVENTTRADINGREWARDDISTRIBUTION._serialized_end=4098
  _EVENTNEWCONDITIONALDERIVATIVEORDER._serialized_start=4101
  _EVENTNEWCONDITIONALDERIVATIVEORDER._serialized_end=4249
  _EVENTCANCELCONDITIONALDERIVATIVEORDER._serialized_start=4252
  _EVENTCANCELCONDITIONALDERIVATIVEORDER._serialized_end=4489
  _EVENTCONDITIONALDERIVATIVEORDERTRIGGER._serialized_start=4492
  _EVENTCONDITIONALDERIVATIVEORDERTRIGGER._serialized_end=4632
  _EVENTORDERFAIL._serialized_start=4634
  _EVENTORDERFAIL._serialized_end=4698
  _EVENTATOMICMARKETORDERFEEMULTIPLIERSUPDATED._serialized_start=4700
  _EVENTATOMICMARKETORDERFEEMULTIPLIERSUPDATED._serialized_end=4826
  _EVENTORDERBOOKUPDATE._serialized_start=4829
  _EVENTORDERBOOKUPDATE._serialized_end=4991
  _ORDERBOOKUPDATE._serialized_start=4993
  _ORDERBOOKUPDATE._serialized_end=5081
  _ORDERBOOK._serialized_start=5084
  _ORDERBOOK._serialized_end=5225
# @@protoc_insertion_point(module_scope)



import os
from configparser import ConfigParser

MAX_CLIENT_ID_LENGTH = 128
MAX_DATA_SIZE = 256
MAX_MEMO_CHARACTERS = 256

devnet_config = ConfigParser()
devnet_config.read(os.path.join(os.path.dirname(__file__), 'denoms_devnet.ini'))

testnet_config = ConfigParser()
testnet_config.read(os.path.join(os.path.dirname(__file__), 'denoms_testnet.ini'))

mainnet_config = ConfigParser()
mainnet_config.read(os.path.join(os.path.dirname(__file__), 'denoms_mainnet.ini'))

class Denom:
    def __init__(
        self,
        description: str,
        base: int,
        quote: int,
        min_price_tick_size: float,
        min_quantity_tick_size: float
    ):
        self.description = description
        self.base = base
        self.quote = quote
        self.min_price_tick_size = min_price_tick_size
        self.min_quantity_tick_size = min_quantity_tick_size

    @classmethod
    def load_market(cls, network, market_id):
        if network == 'devnet':
            config = devnet_config
        elif network == 'testnet':
            config = testnet_config
        else:
            config =mainnet_config

        return cls(
            description=config[market_id]['description'],
            base=int(config[market_id]['base']),
            quote=int(config[market_id]['quote']),
            min_price_tick_size=float(config[market_id]['min_price_tick_size']),
            min_quantity_tick_size=float(config[market_id]['min_quantity_tick_size']),
        )

    @classmethod
    def load_peggy_denom(cls, network, symbol):
        if network == 'devnet':
            config = devnet_config
        elif network == 'local':
            config = devnet_config
        elif network == 'testnet':
            config = testnet_config
        else:
            config = mainnet_config
        return config[symbol]['peggy_denom'], int(config[symbol]['decimals'])


class Network:
    def __init__(
        self,
        lcd_endpoint: str ,
        tm_websocket_endpoint: str,
        grpc_endpoint: str ,
        grpc_exchange_endpoint: str ,
        grpc_explorer_endpoint: str ,
        chain_id: str ,
        fee_denom: str ,
        env: str
    ):
        self.lcd_endpoint = lcd_endpoint
        self.tm_websocket_endpoint = tm_websocket_endpoint
        self.grpc_endpoint = grpc_endpoint
        self.grpc_exchange_endpoint = grpc_exchange_endpoint
        self.grpc_explorer_endpoint = grpc_explorer_endpoint
        self.chain_id = chain_id
        self.fee_denom = fee_denom
        self.env = env

    @classmethod
    def devnet(cls):
        return cls(
            lcd_endpoint='https://devnet.lcd.injective.dev',
            tm_websocket_endpoint='wss://devnet.tm.injective.dev/websocket',
            grpc_endpoint='devnet.injective.dev:9900',
            grpc_exchange_endpoint='devnet.injective.dev:9910',
            grpc_explorer_endpoint='devnet.injective.dev:9911',
            chain_id='injective-777',
            fee_denom='inj',
            env='devnet'
        )

    @classmethod
    def testnet(cls):
        return cls(
            lcd_endpoint='https://k8s.testnet.lcd.injective.network',
            tm_websocket_endpoint='wss://k8s.testnet.tm.injective.network/websocket',
            grpc_endpoint='k8s.testnet.chain.grpc.injective.network:443',
            grpc_exchange_endpoint='k8s.testnet.exchange.grpc.injective.network:443',
            grpc_explorer_endpoint='k8s.testnet.explorer.grpc.injective.network:443',
            chain_id='injective-888',
            fee_denom='inj',
            env='testnet'
        )

    @classmethod
    def mainnet(cls, node='lb'):
        nodes = [
            'k8s', # us, prod
            'lb', # us, asia, prod
            'sentry0',  # ca, prod
            'sentry1',  # ca, prod
            'sentry3',  # us, prod
        ]
        if node not in nodes:
            raise ValueError('Must be one of {}'.format(nodes))

        if node == 'lb':
            lcd_endpoint = 'https://k8s.global.mainnet.lcd.injective.network:443'
            tm_websocket_endpoint = 'wss://k8s.global.mainnet.tm.injective.network:443/websocket'
            grpc_endpoint = 'k8s.global.mainnet.chain.grpc.injective.network:443'
            grpc_exchange_endpoint = 'k8s.global.mainnet.exchange.grpc.injective.network:443'
            grpc_explorer_endpoint = 'k8s.global.mainnet.explorer.grpc.injective.network:443'
        else:
            lcd_endpoint='https://lcd.injective.network'
            tm_websocket_endpoint=f'ws://{node}.injective.network:26657/websocket'
            grpc_endpoint=f'{node}.injective.network:9900'
            grpc_exchange_endpoint=f'{node}.injective.network:9910'
            grpc_explorer_endpoint=f'{node}.injective.network:9911'

        return cls(
            lcd_endpoint=lcd_endpoint,
            tm_websocket_endpoint=tm_websocket_endpoint,
            grpc_endpoint=grpc_endpoint,
            grpc_exchange_endpoint=grpc_exchange_endpoint,
            grpc_explorer_endpoint=grpc_explorer_endpoint,
            chain_id='injective-1',
            fee_denom='inj',
            env='mainnet'
        )

    @classmethod
    def local(cls):
        return cls(
            lcd_endpoint='http://localhost:10337',
            tm_websocket_endpoint='ws://localhost:26657/websocket',
            grpc_endpoint='localhost:9900',
            grpc_exchange_endpoint='localhost:9910',
            grpc_explorer_endpoint='localhost:9911',
            chain_id='injective-1',
            fee_denom='inj',
            env='local'
        )

    @classmethod
    def custom(cls, lcd_endpoint, tm_websocket_endpoint, grpc_endpoint, grpc_exchange_endpoint, grpc_explorer_endpoint, chain_id, env):
        return cls(
            lcd_endpoint=lcd_endpoint,
            tm_websocket_endpoint=tm_websocket_endpoint,
            grpc_endpoint=grpc_endpoint,
            grpc_exchange_endpoint=grpc_exchange_endpoint,
            grpc_explorer_endpoint=grpc_explorer_endpoint,
            chain_id=chain_id,
            fee_denom='inj',
            env=env
        )

    def string(self):
        return self.env



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: injective/exchange/v1beta1/exchange.proto
"""Generated protocol buffer code."""
from google.protobuf.internal import builder as _builder
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from gogoproto import gogo_pb2 as gogoproto_dot_gogo__pb2
from cosmos.base.v1beta1 import coin_pb2 as cosmos_dot_base_dot_v1beta1_dot_coin__pb2
from injective.oracle.v1beta1 import oracle_pb2 as injective_dot_oracle_dot_v1beta1_dot_oracle__pb2


DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n)injective/exchange/v1beta1/exchange.proto\x12\x1ainjective.exchange.v1beta1\x1a\x14gogoproto/gogo.proto\x1a\x1e\x63osmos/base/v1beta1/coin.proto\x1a%injective/oracle/v1beta1/oracle.proto\"\xf0\x0e\n\x06Params\x12H\n\x1fspot_market_instant_listing_fee\x18\x01 \x01(\x0b\x32\x19.cosmos.base.v1beta1.CoinB\x04\xc8\xde\x1f\x00\x12N\n%derivative_market_instant_listing_fee\x18\x02 \x01(\x0b\x32\x19.cosmos.base.v1beta1.CoinB\x04\xc8\xde\x1f\x00\x12S\n\x1b\x64\x65\x66\x61ult_spot_maker_fee_rate\x18\x03 \x01(\tB.\xda\xde\x1f&github.com/cosmos/cosmos-sdk/types.Dec\xc8\xde\x1f\x00\x12S\n\x1b\x64\x65\x66\x61ult_spot_taker_fee_rate\x18\x04 \x01(\tB.\xda\xde\x1f&github.com/cosmos/cosmos-sdk/types.Dec\xc8\xde\x1f\x00\x12Y\n!default_derivative_maker_fee_rate\x18\x05 \x01(\tB.\xda\xde\x1f&github.com/cosmos/cosmos-sdk/types.Dec\xc8\xde\x1f\x00\x12Y\n!default_derivative_taker_fee_rate\x18\x06 \x01(\tB.\xda\xde\x1f&github.com/cosmos/cosmos-sdk/types.Dec\xc8\xde\x1f\x00\x12T\n\x1c\x64\x65\x66\x61ult_initial_margin_ratio\x18\x07 \x01(\tB.\xda\xde\x1f&github.com/cosmos/cosmos-sdk/types.Dec\xc8\xde\x1f\x00\x12X\n default_maintenance_margin_ratio\x18\x08 \x01(\tB.\xda\xde\x1f&github.com/cosmos/cosmos-sdk/types.Dec\xc8\xde\x1f\x00\x12 \n\x18\x64\x65\x66\x61ult_funding_interval\x18\t \x01(\x03\x12\x18\n\x10\x66unding_multiple\x18\n \x01(\x03\x12N\n\x16relayer_fee_share_rate\x18\x0b \x01(\tB.\xda\xde\x1f&github.com/cosmos/cosmos-sdk/types.Dec\xc8\xde\x1f\x00\x12W\n\x1f\x64\x65\x66\x61ult_hourly_funding_rate_cap\x18\x0c \x01(\tB.\xda\xde\x1f&github.com/cosmos/cosmos-sdk/types.Dec\xc8\xde\x1f\x00\x12T\n\x1c\x64\x65\x66\x61ult_hourly_interest_rate\x18\r \x01(\tB.\xda\xde\x1f&github.com/cosmos/cosmos-sdk/types.Dec\xc8\xde\x1f\x00\x12\'\n\x1fmax_derivative_order_side_count\x18\x0e \x01(\r\x12_\n\'inj_reward_staked_requirement_threshold\x18\x0f \x01(\tB.\xda\xde\x1f&github.com/cosmos/cosmos-sdk/types.Int\xc8\xde\x1f\x00\x12(\n trading_rewards_vesting_duration\x18\x10 \x01(\x03\x12T\n\x1cliquidator_reward_share_rate\x18\x11 \x01(\tB.\xda\xde\x1f&github.com/cosmos/cosmos-sdk/types.Dec\xc8\xde\x1f\x00\x12R\n)binary_options_market_instant_listing_fee\x18\x12 \x01(\x0b\x32\x19.cosmos.base.v1beta1.CoinB\x04\xc8\xde\x1f\x00\x12\x62\n atomic_market_order_access_level\x18\x13 \x01(\x0e\x32\x38.injective.exchange.v1beta1.AtomicMarketOrderAccessLevel\x12_\n\'spot_atomic_market_order_fee_multiplier\x18\x14 \x01(\tB.\xda\xde\x1f&github.com/cosmos/cosmos-sdk/types.Dec\xc8\xde\x1f\x00\x12\x65\n-derivative_atomic_market_order_fee_multiplier\x18\x15 \x01(\tB.\xda\xde\x1f&github.com/cosmos/cosmos-sdk/types.Dec\xc8\xde\x1f\x00\x12i\n1binary_options_atomic_market_order_fee_multiplier\x18\x16 \x01(\tB.\xda\xde\x1f&github.com/cosmos/cosmos-sdk/types.Dec\xc8\xde\x1f\x00\x12Q\n\x19minimal_protocol_fee_rate\x18\x17 \x01(\tB.\xda\xde\x1f&github.com/cosmos/cosmos-sdk/types.Dec\xc8\xde\x1f\x00\x12\x33\n+is_instant_derivative_market_launch_enabled\x18\x18 \x01(\x08:\x04\xe8\xa0\x1f\x01\"v\n\x13MarketFeeMultiplier\x12\x11\n\tmarket_id\x18\x01 \x01(\t\x12\x46\n\x0e\x66\x65\x65_multiplier\x18\x02 \x01(\tB.\xda\xde\x1f&github.com/cosmos/cosmos-sdk/types.Dec\xc8\xde\x1f\x00:\x04\x88\xa0\x1f\x00\"\xbf\x06\n\x10\x44\x65rivativeMarket\x12\x0e\n\x06ticker\x18\x01 \x01(\t\x12\x13\n\x0boracle_base\x18\x02 \x01(\t\x12\x14\n\x0coracle_quote\x18\x03 \x01(\t\x12\x39\n\x0boracle_type\x18\x04 \x01(\x0e\x32$.injective.oracle.v1beta1.OracleType\x12\x1b\n\x13oracle_scale_factor\x18\x05 \x01(\r\x12\x13\n\x0bquote_denom\x18\x06 \x01(\t\x12\x11\n\tmarket_id\x18\x07 \x01(\t\x12L\n\x14initial_margin_ratio\x18\x08 \x01(\tB.\xda\xde\x1f&github.com/cosmos/cosmos-sdk/types.Dec\xc8\xde\x1f\x00\x12P\n\x18maintenance_margin_ratio\x18\t \x01(\tB.\xda\xde\x1f&github.com/cosmos/cosmos-sdk/types.Dec\xc8\xde\x1f\x00\x12\x46\n\x0emaker_fee_rate\x18\n \x01(\tB.\xda\xde\x1f&github.com/cosmos/cosmos-sdk/types.Dec\xc8\xde\x1f\x00\x12\x46\n\x0etaker_fee_rate\x18\x0b \x01(\tB.\xda\xde\x1f&github.com/cosmos/cosmos-sdk/types.Dec\xc8\xde\x1f\x00\x12N\n\x16relayer_fee_share_rate\x18\x0c \x01(\tB.\xda\xde\x1f&github.com/cosmos/cosmos-sdk/types.Dec\xc8\xde\x1f\x00\x12\x13\n\x0bisPerpetual\x18\r \x01(\x08\x12\x38\n\x06status\x18\x0e \x01(\x0e\x32(.injective.exchange.v1beta1.MarketStatus\x12K\n\x13min_price_tick_size\x18\x0f \x01(\tB.\xda\xde\x1f&github.com/cosmos/cosmos-sdk/types.Dec\xc8\xde\x1f\x00\x12N\n\x16min_quantity_tick_size\x18\x10 \x01(\tB.\xda\xde\x1f&github.com/cosmos/cosmos-sdk/types.Dec\xc8\xde\x1f\x00:\x04\x88\xa0\x1f\x00\"\xa7\x06\n\x13\x42inaryOptionsMarket\x12\x0e\n\x06ticker\x18\x01 \x01(\t\x12\x15\n\roracle_symbol\x18\x02 \x01(\t\x12\x17\n\x0foracle_provider\x18\x03 \x01(\t\x12\x39\n\x0boracle_type\x18\x04 \x01(\x0e\x32$.injective.oracle.v1beta1.OracleType\x12\x1b\n\x13oracle_scale_factor\x18\x05 \x01(\r\x12\x1c\n\x14\x65xpiration_timestamp\x18\x06 \x01(\x03\x12\x1c\n\x14settlement_timestamp\x18\x07 \x01(\x03\x12\r\n\x05\x61\x64min\x18\x08 \x01(\t\x12\x13\n\x0bquote_denom\x18\t \x01(\t\x12\x11\n\tmarket_id\x18\n \x01(\t\x12\x46\n\x0emaker_fee_rate\x18\x0b \x01(\tB.\xda\xde\x1f&github.com/cosmos/cosmos-sdk/types.Dec\xc8\xde\x1f\x00\x12\x46\n\x0etaker_fee_rate\x18\x0c \x01(\tB.\xda\xde\x1f&github.com/cosmos/cosmos-sdk/types.Dec\xc8\xde\x1f\x00\x12N\n\x16relayer_fee_share_rate\x18\r \x01(\tB.\xda\xde\x1f&github.com/cosmos/cosmos-sdk/types.Dec\xc8\xde\x1f\x00\x12\x38\n\x06status\x18\x0e \x01(\x0e\x32(.injective.exchange.v1beta1.MarketStatus\x12K\n\x13min_price_tick_size\x18\x0f \x01(\tB.\xda\xde\x1f&github.com/cosmos/cosmos-sdk/types.Dec\xc8\xde\x1f\x00\x12N\n\x16min_quantity_tick_size\x18\x10 \x01(\tB.\xda\xde\x1f&github.com/cosmos/cosmos-sdk/types.Dec\xc8\xde\x1f\x00\x12H\n\x10settlement_price\x18\x11 \x01(\tB.\xda\xde\x1f&github.com/cosmos/cosmos-sdk/types.Dec\xc8\xde\x1f\x01:\x04\x88\xa0\x1f\x00\"\x92\x02\n\x17\x45xpiryFuturesMarketInfo\x12\x11\n\tmarket_id\x18\x01 \x01(\t\x12\x1c\n\x14\x65xpiration_timestamp\x18\x02 \x01(\x03\x12\x1c\n\x14twap_start_timestamp\x18\x03 \x01(\x03\x12^\n&expiration_twap_start_price_cumulative\x18\x04 \x01(\tB.\xda\xde\x1f&github.com/cosmos/cosmos-sdk/types.Dec\xc8\xde\x1f\x00\x12H\n\x10settlement_price\x18\x05 \x01(\tB.\xda\xde\x1f&github.com/cosmos/cosmos-sdk/types.Dec\xc8\xde\x1f\x00\"\x81\x02\n\x13PerpetualMarketInfo\x12\x11\n\tmarket_id\x18\x01 \x01(\t\x12O\n\x17hourly_funding_rate_cap\x18\x02 \x01(\tB.\xda\xde\x1f&github.com/cosmos/cosmos-sdk/types.Dec\xc8\xde\x1f\x00\x12L\n\x14hourly_interest_rate\x18\x03 \x01(\tB.\xda\xde\x1f&github.com/cosmos/cosmos-sdk/types.Dec\xc8\xde\x1f\x00\x12\x1e\n\x16next_funding_timestamp\x18\x04 \x01(\x03\x12\x18\n\x10\x66unding_interval\x18\x05 \x01(\x03\"\xc6\x01\n\x16PerpetualMarketFunding\x12J\n\x12\x63umulative_funding\x18\x01 \x01(\tB.\xda\xde\x1f&github.com/cosmos/cosmos-sdk/types.Dec\xc8\xde\x1f\x00\x12H\n\x10\x63umulative_price\x18\x02 \x01(\tB.\xda\xde\x1f&github.com/cosmos/cosmos-sdk/types.Dec\xc8\xde\x1f\x00\x12\x16\n\x0elast_timestamp\x18\x03 \x01(\x03\"}\n\x1e\x44\x65rivativeMarketSettlementInfo\x12\x11\n\tmarket_id\x18\x01 \x01(\t\x12H\n\x10settlement_price\x18\x02 \x01(\tB.\xda\xde\x1f&github.com/cosmos/cosmos-sdk/types.Dec\xc8\xde\x1f\x00\".\n\x14NextFundingTimestamp\x12\x16\n\x0enext_timestamp\x18\x01 \x01(\x03\"\x8f\x04\n\nSpotMarket\x12\x0e\n\x06ticker\x18\x01 \x01(\t\x12\x12\n\nbase_denom\x18\x02 \x01(\t\x12\x13\n\x0bquote_denom\x18\x03 \x01(\t\x12\x46\n\x0emaker_fee_rate\x18\x04 \x01(\tB.\xda\xde\x1f&github.com/cosmos/cosmos-sdk/types.Dec\xc8\xde\x1f\x00\x12\x46\n\x0etaker_fee_rate\x18\x05 \x01(\tB.\xda\xde\x1f&github.com/cosmos/cosmos-sdk/types.Dec\xc8\xde\x1f\x00\x12N\n\x16relayer_fee_share_rate\x18\x06 \x01(\tB.\xda\xde\x1f&github.com/cosmos/cosmos-sdk/types.Dec\xc8\xde\x1f\x00\x12\x11\n\tmarket_id\x18\x07 \x01(\t\x12\x38\n\x06status\x18\x08 \x01(\x0e\x32(.injective.exchange.v1beta1.MarketStatus\x12K\n\x13min_price_tick_size\x18\t \x01(\tB.\xda\xde\x1f&github.com/cosmos/cosmos-sdk/types.Dec\xc8\xde\x1f\x00\x12N\n\x16min_quantity_tick_size\x18\n \x01(\tB.\xda\xde\x1f&github.com/cosmos/cosmos-sdk/types.Dec\xc8\xde\x1f\x00\"\x9b\x01\n\x07\x44\x65posit\x12I\n\x11\x61vailable_balance\x18\x01 \x01(\tB.\xda\xde\x1f&github.com/cosmos/cosmos-sdk/types.Dec\xc8\xde\x1f\x00\x12\x45\n\rtotal_balance\x18\x02 \x01(\tB.\xda\xde\x1f&github.com/cosmos/cosmos-sdk/types.Dec\xc8\xde\x1f\x00\"%\n\x14SubaccountTradeNonce\x12\r\n\x05nonce\x18\x01 \x01(\r\"\xba\x01\n\tOrderInfo\x12\x15\n\rsubaccount_id\x18\x01 \x01(\t\x12\x15\n\rfee_recipient\x18\x02 \x01(\t\x12=\n\x05price\x18\x03 \x01(\tB.\xda\xde\x1f&github.com/cosmos/cosmos-sdk/types.Dec\xc8\xde\x1f\x00\x12@\n\x08quantity\x18\x04 \x01(\tB.\xda\xde\x1f&github.com/cosmos/cosmos-sdk/types.Dec\xc8\xde\x1f\x00\"\xe1\x01\n\tSpotOrder\x12\x11\n\tmarket_id\x18\x01 \x01(\t\x12?\n\norder_info\x18\x02 \x01(\x0b\x32%.injective.exchange.v1beta1.OrderInfoB\x04\xc8\xde\x1f\x00\x12\x39\n\norder_type\x18\x03 \x01(\x0e\x32%.injective.exchange.v1beta1.OrderType\x12\x45\n\rtrigger_price\x18\x04 \x01(\tB.\xda\xde\x1f&github.com/cosmos/cosmos-sdk/types.Dec\xc8\xde\x1f\x01\"\xa9\x02\n\x0eSpotLimitOrder\x12?\n\norder_info\x18\x01 \x01(\x0b\x32%.injective.exchange.v1beta1.OrderInfoB\x04\xc8\xde\x1f\x00\x12\x39\n\norder_type\x18\x02 \x01(\x0e\x32%.injective.exchange.v1beta1.OrderType\x12@\n\x08\x66illable\x18\x03 \x01(\tB.\xda\xde\x1f&github.com/cosmos/cosmos-sdk/types.Dec\xc8\xde\x1f\x00\x12\x45\n\rtrigger_price\x18\x04 \x01(\tB.\xda\xde\x1f&github.com/cosmos/cosmos-sdk/types.Dec\xc8\xde\x1f\x01\x12\x12\n\norder_hash\x18\x05 \x01(\x0c\"\xae\x02\n\x0fSpotMarketOrder\x12?\n\norder_info\x18\x01 \x01(\x0b\x32%.injective.exchange.v1beta1.OrderInfoB\x04\xc8\xde\x1f\x00\x12\x44\n\x0c\x62\x61lance_hold\x18\x02 \x01(\tB.\xda\xde\x1f&github.com/cosmos/cosmos-sdk/types.Dec\xc8\xde\x1f\x00\x12\x12\n\norder_hash\x18\x03 \x01(\x0c\x12\x39\n\norder_type\x18\x04 \x01(\x0e\x32%.injective.exchange.v1beta1.OrderType\x12\x45\n\rtrigger_price\x18\x05 \x01(\tB.\xda\xde\x1f&github.com/cosmos/cosmos-sdk/types.Dec\xc8\xde\x1f\x01\"\xa7\x02\n\x0f\x44\x65rivativeOrder\x12\x11\n\tmarket_id\x18\x01 \x01(\t\x12?\n\norder_info\x18\x02 \x01(\x0b\x32%.injective.exchange.v1beta1.OrderInfoB\x04\xc8\xde\x1f\x00\x12\x39\n\norder_type\x18\x03 \x01(\x0e\x32%.injective.exchange.v1beta1.OrderType\x12>\n\x06margin\x18\x04 \x01(\tB.\xda\xde\x1f&github.com/cosmos/cosmos-sdk/types.Dec\xc8\xde\x1f\x00\x12\x45\n\rtrigger_price\x18\x05 \x01(\tB.\xda\xde\x1f&github.com/cosmos/cosmos-sdk/types.Dec\xc8\xde\x1f\x01\"\xe9\x02\n\x1bSubaccountOrderbookMetadata\x12!\n\x19vanilla_limit_order_count\x18\x01 \x01(\r\x12%\n\x1dreduce_only_limit_order_count\x18\x02 \x01(\r\x12V\n\x1e\x61ggregate_reduce_only_quantity\x18\x03 \x01(\tB.\xda\xde\x1f&github.com/cosmos/cosmos-sdk/types.Dec\xc8\xde\x1f\x00\x12R\n\x1a\x61ggregate_vanilla_quantity\x18\x04 \x01(\tB.\xda\xde\x1f&github.com/cosmos/cosmos-sdk/types.Dec\xc8\xde\x1f\x00\x12\'\n\x1fvanilla_conditional_order_count\x18\x05 \x01(\r\x12+\n#reduce_only_conditional_order_count\x18\x06 \x01(\r\"\xa8\x01\n\x0fSubaccountOrder\x12=\n\x05price\x18\x01 \x01(\tB.\xda\xde\x1f&github.com/cosmos/cosmos-sdk/types.Dec\xc8\xde\x1f\x00\x12@\n\x08quantity\x18\x02 \x01(\tB.\xda\xde\x1f&github.com/cosmos/cosmos-sdk/types.Dec\xc8\xde\x1f\x00\x12\x14\n\x0cisReduceOnly\x18\x03 \x01(\x08\"e\n\x13SubaccountOrderData\x12:\n\x05order\x18\x01 \x01(\x0b\x32+.injective.exchange.v1beta1.SubaccountOrder\x12\x12\n\norder_hash\x18\x02 \x01(\x0c\"\xef\x02\n\x14\x44\x65rivativeLimitOrder\x12?\n\norder_info\x18\x01 \x01(\x0b\x32%.injective.exchange.v1beta1.OrderInfoB\x04\xc8\xde\x1f\x00\x12\x39\n\norder_type\x18\x02 \x01(\x0e\x32%.injective.exchange.v1beta1.OrderType\x12>\n\x06margin\x18\x03 \x01(\tB.\xda\xde\x1f&github.com/cosmos/cosmos-sdk/types.Dec\xc8\xde\x1f\x00\x12@\n\x08\x66illable\x18\x04 \x01(\tB.\xda\xde\x1f&github.com/cosmos/cosmos-sdk/types.Dec\xc8\xde\x1f\x00\x12\x45\n\rtrigger_price\x18\x05 \x01(\tB.\xda\xde\x1f&github.com/cosmos/cosmos-sdk/types.Dec\xc8\xde\x1f\x01\x12\x12\n\norder_hash\x18\x06 \x01(\x0c\"\xf3\x02\n\x15\x44\x65rivativeMarketOrder\x12?\n\norder_info\x18\x01 \x01(\x0b\x32%.injective.exchange.v1beta1.OrderInfoB\x04\xc8\xde\x1f\x00\x12\x39\n\norder_type\x18\x02 \x01(\x0e\x32%.injective.exchange.v1beta1.OrderType\x12>\n\x06margin\x18\x03 \x01(\tB.\xda\xde\x1f&github.com/cosmos/cosmos-sdk/types.Dec\xc8\xde\x1f\x00\x12\x43\n\x0bmargin_hold\x18\x04 \x01(\tB.\xda\xde\x1f&github.com/cosmos/cosmos-sdk/types.Dec\xc8\xde\x1f\x00\x12\x45\n\rtrigger_price\x18\x05 \x01(\tB.\xda\xde\x1f&github.com/cosmos/cosmos-sdk/types.Dec\xc8\xde\x1f\x01\x12\x12\n\norder_hash\x18\x06 \x01(\x0c\"\xb3\x02\n\x08Position\x12\x0e\n\x06isLong\x18\x01 \x01(\x08\x12@\n\x08quantity\x18\x02 \x01(\tB.\xda\xde\x1f&github.com/cosmos/cosmos-sdk/types.Dec\xc8\xde\x1f\x00\x12\x43\n\x0b\x65ntry_price\x18\x03 \x01(\tB.\xda\xde\x1f&github.com/cosmos/cosmos-sdk/types.Dec\xc8\xde\x1f\x00\x12>\n\x06margin\x18\x04 \x01(\tB.\xda\xde\x1f&github.com/cosmos/cosmos-sdk/types.Dec\xc8\xde\x1f\x00\x12P\n\x18\x63umulative_funding_entry\x18\x05 \x01(\tB.\xda\xde\x1f&github.com/cosmos/cosmos-sdk/types.Dec\xc8\xde\x1f\x00\"8\n\x14MarketOrderIndicator\x12\x11\n\tmarket_id\x18\x01 \x01(\t\x12\r\n\x05isBuy\x18\x02 \x01(\x08\"\x98\x02\n\x08TradeLog\x12@\n\x08quantity\x18\x01 \x01(\tB.\xda\xde\x1f&github.com/cosmos/cosmos-sdk/types.Dec\xc8\xde\x1f\x00\x12=\n\x05price\x18\x02 \x01(\tB.\xda\xde\x1f&github.com/cosmos/cosmos-sdk/types.Dec\xc8\xde\x1f\x00\x12\x15\n\rsubaccount_id\x18\x03 \x01(\x0c\x12;\n\x03\x66\x65\x65\x18\x04 \x01(\tB.\xda\xde\x1f&github.com/cosmos/cosmos-sdk/types.Dec\xc8\xde\x1f\x00\x12\x12\n\norder_hash\x18\x05 \x01(\x0c\x12#\n\x15\x66\x65\x65_recipient_address\x18\x06 \x01(\x0c\x42\x04\xc8\xde\x1f\x01\"\xff\x01\n\rPositionDelta\x12\x0f\n\x07is_long\x18\x01 \x01(\x08\x12J\n\x12\x65xecution_quantity\x18\x02 \x01(\tB.\xda\xde\x1f&github.com/cosmos/cosmos-sdk/types.Dec\xc8\xde\x1f\x00\x12H\n\x10\x65xecution_margin\x18\x03 \x01(\tB.\xda\xde\x1f&github.com/cosmos/cosmos-sdk/types.Dec\xc8\xde\x1f\x00\x12G\n\x0f\x65xecution_price\x18\x04 \x01(\tB.\xda\xde\x1f&github.com/cosmos/cosmos-sdk/types.Dec\xc8\xde\x1f\x00\"\xa4\x02\n\x12\x44\x65rivativeTradeLog\x12\x15\n\rsubaccount_id\x18\x01 \x01(\x0c\x12\x41\n\x0eposition_delta\x18\x02 \x01(\x0b\x32).injective.exchange.v1beta1.PositionDelta\x12>\n\x06payout\x18\x03 \x01(\tB.\xda\xde\x1f&github.com/cosmos/cosmos-sdk/types.Dec\xc8\xde\x1f\x00\x12;\n\x03\x66\x65\x65\x18\x04 \x01(\tB.\xda\xde\x1f&github.com/cosmos/cosmos-sdk/types.Dec\xc8\xde\x1f\x00\x12\x12\n\norder_hash\x18\x05 \x01(\x0c\x12#\n\x15\x66\x65\x65_recipient_address\x18\x06 \x01(\x0c\x42\x04\xc8\xde\x1f\x01\"c\n\x12SubaccountPosition\x12\x36\n\x08position\x18\x01 \x01(\x0b\x32$.injective.exchange.v1beta1.Position\x12\x15\n\rsubaccount_id\x18\x02 \x01(\x0c\"`\n\x11SubaccountDeposit\x12\x15\n\rsubaccount_id\x18\x01 \x01(\x0c\x12\x34\n\x07\x64\x65posit\x18\x02 \x01(\x0b\x32#.injective.exchange.v1beta1.Deposit\"_\n\rDepositUpdate\x12\r\n\x05\x64\x65nom\x18\x01 \x01(\t\x12?\n\x08\x64\x65posits\x18\x02 \x03(\x0b\x32-.injective.exchange.v1beta1.SubaccountDeposit\"\xb4\x01\n\x10PointsMultiplier\x12O\n\x17maker_points_multiplier\x18\x01 \x01(\tB.\xda\xde\x1f&github.com/cosmos/cosmos-sdk/types.Dec\xc8\xde\x1f\x00\x12O\n\x17taker_points_multiplier\x18\x02 \x01(\tB.\xda\xde\x1f&github.com/cosmos/cosmos-sdk/types.Dec\xc8\xde\x1f\x00\"\x98\x02\n\x1eTradingRewardCampaignBoostInfo\x12\x1f\n\x17\x62oosted_spot_market_ids\x18\x01 \x03(\t\x12S\n\x17spot_market_multipliers\x18\x02 \x03(\x0b\x32,.injective.exchange.v1beta1.PointsMultiplierB\x04\xc8\xde\x1f\x00\x12%\n\x1d\x62oosted_derivative_market_ids\x18\x03 \x03(\t\x12Y\n\x1d\x64\x65rivative_market_multipliers\x18\x04 \x03(\x0b\x32,.injective.exchange.v1beta1.PointsMultiplierB\x04\xc8\xde\x1f\x00\"\x98\x01\n\x12\x43\x61mpaignRewardPool\x12\x17\n\x0fstart_timestamp\x18\x01 \x01(\x03\x12i\n\x14max_campaign_rewards\x18\x02 \x03(\x0b\x32\x19.cosmos.base.v1beta1.CoinB0\xc8\xde\x1f\x00\xaa\xdf\x1f(github.com/cosmos/cosmos-sdk/types.Coins\"\xd4\x01\n\x19TradingRewardCampaignInfo\x12!\n\x19\x63\x61mpaign_duration_seconds\x18\x01 \x01(\x03\x12\x14\n\x0cquote_denoms\x18\x02 \x03(\t\x12]\n\x19trading_reward_boost_info\x18\x03 \x01(\x0b\x32:.injective.exchange.v1beta1.TradingRewardCampaignBoostInfo\x12\x1f\n\x17\x64isqualified_market_ids\x18\x04 \x03(\t\"\xb6\x02\n\x13\x46\x65\x65\x44iscountTierInfo\x12K\n\x13maker_discount_rate\x18\x01 \x01(\tB.\xda\xde\x1f&github.com/cosmos/cosmos-sdk/types.Dec\xc8\xde\x1f\x00\x12K\n\x13taker_discount_rate\x18\x02 \x01(\tB.\xda\xde\x1f&github.com/cosmos/cosmos-sdk/types.Dec\xc8\xde\x1f\x00\x12\x45\n\rstaked_amount\x18\x03 \x01(\tB.\xda\xde\x1f&github.com/cosmos/cosmos-sdk/types.Int\xc8\xde\x1f\x00\x12>\n\x06volume\x18\x04 \x01(\tB.\xda\xde\x1f&github.com/cosmos/cosmos-sdk/types.Dec\xc8\xde\x1f\x00\"\xc0\x01\n\x13\x46\x65\x65\x44iscountSchedule\x12\x14\n\x0c\x62ucket_count\x18\x01 \x01(\x04\x12\x17\n\x0f\x62ucket_duration\x18\x02 \x01(\x03\x12\x14\n\x0cquote_denoms\x18\x03 \x03(\t\x12\x43\n\ntier_infos\x18\x04 \x03(\x0b\x32/.injective.exchange.v1beta1.FeeDiscountTierInfo\x12\x1f\n\x17\x64isqualified_market_ids\x18\x05 \x03(\t\"9\n\x12\x46\x65\x65\x44iscountTierTTL\x12\x0c\n\x04tier\x18\x01 \x01(\x04\x12\x15\n\rttl_timestamp\x18\x02 \x01(\x03\"\x9a\x01\n\x0cVolumeRecord\x12\x44\n\x0cmaker_volume\x18\x01 \x01(\tB.\xda\xde\x1f&github.com/cosmos/cosmos-sdk/types.Dec\xc8\xde\x1f\x00\x12\x44\n\x0ctaker_volume\x18\x02 \x01(\tB.\xda\xde\x1f&github.com/cosmos/cosmos-sdk/types.Dec\xc8\xde\x1f\x00\"\x7f\n\x0e\x41\x63\x63ountRewards\x12\x0f\n\x07\x61\x63\x63ount\x18\x01 \x01(\t\x12\\\n\x07rewards\x18\x02 \x03(\x0b\x32\x19.cosmos.base.v1beta1.CoinB0\xc8\xde\x1f\x00\xaa\xdf\x1f(github.com/cosmos/cosmos-sdk/types.Coins\"h\n\x0cTradeRecords\x12\x11\n\tmarket_id\x18\x01 \x01(\t\x12\x45\n\x14latest_trade_records\x18\x02 \x03(\x0b\x32\'.injective.exchange.v1beta1.TradeRecord\"\'\n\rSubaccountIDs\x12\x16\n\x0esubaccount_ids\x18\x01 \x03(\x0c\"\xa1\x01\n\x0bTradeRecord\x12\x11\n\ttimestamp\x18\x01 \x01(\x03\x12=\n\x05price\x18\x02 \x01(\tB.\xda\xde\x1f&github.com/cosmos/cosmos-sdk/types.Dec\xc8\xde\x1f\x00\x12@\n\x08quantity\x18\x03 \x01(\tB.\xda\xde\x1f&github.com/cosmos/cosmos-sdk/types.Dec\xc8\xde\x1f\x00\"}\n\x05Level\x12\x39\n\x01p\x18\x01 \x01(\tB.\xda\xde\x1f&github.com/cosmos/cosmos-sdk/types.Dec\xc8\xde\x1f\x00\x12\x39\n\x01q\x18\x02 \x01(\tB.\xda\xde\x1f&github.com/cosmos/cosmos-sdk/types.Dec\xc8\xde\x1f\x00\"z\n\x1f\x41ggregateSubaccountVolumeRecord\x12\x15\n\rsubaccount_id\x18\x01 \x01(\t\x12@\n\x0emarket_volumes\x18\x02 \x03(\x0b\x32(.injective.exchange.v1beta1.MarketVolume\"q\n\x1c\x41ggregateAccountVolumeRecord\x12\x0f\n\x07\x61\x63\x63ount\x18\x01 \x01(\t\x12@\n\x0emarket_volumes\x18\x02 \x03(\x0b\x32(.injective.exchange.v1beta1.MarketVolume\"a\n\x0cMarketVolume\x12\x11\n\tmarket_id\x18\x01 \x01(\t\x12>\n\x06volume\x18\x02 \x01(\x0b\x32(.injective.exchange.v1beta1.VolumeRecordB\x04\xc8\xde\x1f\x00\"0\n\rDenomDecimals\x12\r\n\x05\x64\x65nom\x18\x01 \x01(\t\x12\x10\n\x08\x64\x65\x63imals\x18\x02 \x01(\x04*t\n\x1c\x41tomicMarketOrderAccessLevel\x12\n\n\x06Nobody\x10\x00\x12\"\n\x1e\x42\x65ginBlockerSmartContractsOnly\x10\x01\x12\x16\n\x12SmartContractsOnly\x10\x02\x12\x0c\n\x08\x45veryone\x10\x03*T\n\x0cMarketStatus\x12\x0f\n\x0bUnspecified\x10\x00\x12\n\n\x06\x41\x63tive\x10\x01\x12\n\n\x06Paused\x10\x02\x12\x0e\n\nDemolished\x10\x03\x12\x0b\n\x07\x45xpired\x10\x04*\xbb\x02\n\tOrderType\x12 \n\x0bUNSPECIFIED\x10\x00\x1a\x0f\x8a\x9d \x0bUNSPECIFIED\x12\x10\n\x03\x42UY\x10\x01\x1a\x07\x8a\x9d \x03\x42UY\x12\x12\n\x04SELL\x10\x02\x1a\x08\x8a\x9d \x04SELL\x12\x1a\n\x08STOP_BUY\x10\x03\x1a\x0c\x8a\x9d \x08STOP_BUY\x12\x1c\n\tSTOP_SELL\x10\x04\x1a\r\x8a\x9d \tSTOP_SELL\x12\x1a\n\x08TAKE_BUY\x10\x05\x1a\x0c\x8a\x9d \x08TAKE_BUY\x12\x1c\n\tTAKE_SELL\x10\x06\x1a\r\x8a\x9d \tTAKE_SELL\x12\x16\n\x06\x42UY_PO\x10\x07\x1a\n\x8a\x9d \x06\x42UY_PO\x12\x18\n\x07SELL_PO\x10\x08\x1a\x0b\x8a\x9d \x07SELL_PO\x12\x1e\n\nBUY_ATOMIC\x10\t\x1a\x0e\x8a\x9d \nBUY_ATOMIC\x12 \n\x0bSELL_ATOMIC\x10\n\x1a\x0f\x8a\x9d \x0bSELL_ATOMIC*\xaf\x01\n\rExecutionType\x12\x1c\n\x18UnspecifiedExecutionType\x10\x00\x12\n\n\x06Market\x10\x01\x12\r\n\tLimitFill\x10\x02\x12\x1a\n\x16LimitMatchRestingOrder\x10\x03\x12\x16\n\x12LimitMatchNewOrder\x10\x04\x12\x15\n\x11MarketLiquidation\x10\x05\x12\x1a\n\x16\x45xpiryMarketSettlement\x10\x06*\x89\x02\n\tOrderMask\x12\x16\n\x06UNUSED\x10\x00\x1a\n\x8a\x9d \x06UNUSED\x12\x10\n\x03\x41NY\x10\x01\x1a\x07\x8a\x9d \x03\x41NY\x12\x18\n\x07REGULAR\x10\x02\x1a\x0b\x8a\x9d \x07REGULAR\x12 \n\x0b\x43ONDITIONAL\x10\x04\x1a\x0f\x8a\x9d \x0b\x43ONDITIONAL\x12.\n\x17\x44IRECTION_BUY_OR_HIGHER\x10\x08\x1a\x11\x8a\x9d \rBUY_OR_HIGHER\x12.\n\x17\x44IRECTION_SELL_OR_LOWER\x10\x10\x1a\x11\x8a\x9d \rSELL_OR_LOWER\x12\x1b\n\x0bTYPE_MARKET\x10 \x1a\n\x8a\x9d \x06MARKET\x12\x19\n\nTYPE_LIMIT\x10@\x1a\t\x8a\x9d \x05LIMITBPZNgithub.com/InjectiveLabs/injective-core/injective-chain/modules/exchange/typesb\x06proto3')

_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, globals())
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'injective.exchange.v1beta1.exchange_pb2', globals())
if _descriptor._USE_C_DESCRIPTORS == False:

  DESCRIPTOR._options = None
  DESCRIPTOR._serialized_options = b'ZNgithub.com/InjectiveLabs/injective-core/injective-chain/modules/exchange/types'
  _ORDERTYPE.values_by_name["UNSPECIFIED"]._options = None
  _ORDERTYPE.values_by_name["UNSPECIFIED"]._serialized_options = b'\212\235 \013UNSPECIFIED'
  _ORDERTYPE.values_by_name["BUY"]._options = None
  _ORDERTYPE.values_by_name["BUY"]._serialized_options = b'\212\235 \003BUY'
  _ORDERTYPE.values_by_name["SELL"]._options = None
  _ORDERTYPE.values_by_name["SELL"]._serialized_options = b'\212\235 \004SELL'
  _ORDERTYPE.values_by_name["STOP_BUY"]._options = None
  _ORDERTYPE.values_by_name["STOP_BUY"]._serialized_options = b'\212\235 \010STOP_BUY'
  _ORDERTYPE.values_by_name["STOP_SELL"]._options = None
  _ORDERTYPE.values_by_name["STOP_SELL"]._serialized_options = b'\212\235 \tSTOP_SELL'
  _ORDERTYPE.values_by_name["TAKE_BUY"]._options = None
  _ORDERTYPE.values_by_name["TAKE_BUY"]._serialized_options = b'\212\235 \010TAKE_BUY'
  _ORDERTYPE.values_by_name["TAKE_SELL"]._options = None
  _ORDERTYPE.values_by_name["TAKE_SELL"]._serialized_options = b'\212\235 \tTAKE_SELL'
  _ORDERTYPE.values_by_name["BUY_PO"]._options = None
  _ORDERTYPE.values_by_name["BUY_PO"]._serialized_options = b'\212\235 \006BUY_PO'
  _ORDERTYPE.values_by_name["SELL_PO"]._options = None
  _ORDERTYPE.values_by_name["SELL_PO"]._serialized_options = b'\212\235 \007SELL_PO'
  _ORDERTYPE.values_by_name["BUY_ATOMIC"]._options = None
  _ORDERTYPE.values_by_name["BUY_ATOMIC"]._serialized_options = b'\212\235 \nBUY_ATOMIC'
  _ORDERTYPE.values_by_name["SELL_ATOMIC"]._options = None
  _ORDERTYPE.values_by_name["SELL_ATOMIC"]._serialized_options = b'\212\235 \013SELL_ATOMIC'
  _ORDERMASK.values_by_name["UNUSED"]._options = None
  _ORDERMASK.values_by_name["UNUSED"]._serialized_options = b'\212\235 \006UNUSED'
  _ORDERMASK.values_by_name["ANY"]._options = None
  _ORDERMASK.values_by_name["ANY"]._serialized_options = b'\212\235 \003ANY'
  _ORDERMASK.values_by_name["REGULAR"]._options = None
  _ORDERMASK.values_by_name["REGULAR"]._serialized_options = b'\212\235 \007REGULAR'
  _ORDERMASK.values_by_name["CONDITIONAL"]._options = None
  _ORDERMASK.values_by_name["CONDITIONAL"]._serialized_options = b'\212\235 \013CONDITIONAL'
  _ORDERMASK.values_by_name["DIRECTION_BUY_OR_HIGHER"]._options = None
  _ORDERMASK.values_by_name["DIRECTION_BUY_OR_HIGHER"]._serialized_options = b'\212\235 \rBUY_OR_HIGHER'
  _ORDERMASK.values_by_name["DIRECTION_SELL_OR_LOWER"]._options = None
  _ORDERMASK.values_by_name["DIRECTION_SELL_OR_LOWER"]._serialized_options = b'\212\235 \rSELL_OR_LOWER'
  _ORDERMASK.values_by_name["TYPE_MARKET"]._options = None
  _ORDERMASK.values_by_name["TYPE_MARKET"]._serialized_options = b'\212\235 \006MARKET'
  _ORDERMASK.values_by_name["TYPE_LIMIT"]._options = None
  _ORDERMASK.values_by_name["TYPE_LIMIT"]._serialized_options = b'\212\235 \005LIMIT'
  _PARAMS.fields_by_name['spot_market_instant_listing_fee']._options = None
  _PARAMS.fields_by_name['spot_market_instant_listing_fee']._serialized_options = b'\310\336\037\000'
  _PARAMS.fields_by_name['derivative_market_instant_listing_fee']._options = None
  _PARAMS.fields_by_name['derivative_market_instant_listing_fee']._serialized_options = b'\310\336\037\000'
  _PARAMS.fields_by_name['default_spot_maker_fee_rate']._options = None
  _PARAMS.fields_by_name['default_spot_maker_fee_rate']._serialized_options = b'\332\336\037&github.com/cosmos/cosmos-sdk/types.Dec\310\336\037\000'
  _PARAMS.fields_by_name['default_spot_taker_fee_rate']._options = None
  _PARAMS.fields_by_name['default_spot_taker_fee_rate']._serialized_options = b'\332\336\037&github.com/cosmos/cosmos-sdk/types.Dec\310\336\037\000'
  _PARAMS.fields_by_name['default_derivative_maker_fee_rate']._options = None
  _PARAMS.fields_by_name['default_derivative_maker_fee_rate']._serialized_options = b'\332\336\037&github.com/cosmos/cosmos-sdk/types.Dec\310\336\037\000'
  _PARAMS.fields_by_name['default_derivative_taker_fee_rate']._options = None
  _PARAMS.fields_by_name['default_derivative_taker_fee_rate']._serialized_options = b'\332\336\037&github.com/cosmos/cosmos-sdk/types.Dec\310\336\037\000'
  _PARAMS.fields_by_name['default_initial_margin_ratio']._options = None
  _PARAMS.fields_by_name['default_initial_margin_ratio']._serialized_options = b'\332\336\037&github.com/cosmos/cosmos-sdk/types.Dec\310\336\037\000'
  _PARAMS.fields_by_name['default_maintenance_margin_ratio']._options = None
  _PARAMS.fields_by_name['default_maintenance_margin_ratio']._serialized_options = b'\332\336\037&github.com/cosmos/cosmos-sdk/types.Dec\310\336\037\000'
  _PARAMS.fields_by_name['relayer_fee_share_rate']._options = None
  _PARAMS.fields_by_name['relayer_fee_share_rate']._serialized_options = b'\332\336\037&github.com/cosmos/cosmos-sdk/types.Dec\310\336\037\000'
  _PARAMS.fields_by_name['default_hourly_funding_rate_cap']._options = None
  _PARAMS.fields_by_name['default_hourly_funding_rate_cap']._serialized_options = b'\332\336\037&github.com/cosmos/cosmos-sdk/types.Dec\310\336\037\000'
  _PARAMS.fields_by_name['default_hourly_interest_rate']._options = None
  _PARAMS.fields_by_name['default_hourly_interest_rate']._serialized_options = b'\332\336\037&github.com/cosmos/cosmos-sdk/types.Dec\310\336\037\000'
  _PARAMS.fields_by_name['inj_reward_staked_requirement_threshold']._options = None
  _PARAMS.fields_by_name['inj_reward_staked_requirement_threshold']._serialized_options = b'\332\336\037&github.com/cosmos/cosmos-sdk/types.Int\310\336\037\000'
  _PARAMS.fields_by_name['liquidator_reward_share_rate']._options = None
  _PARAMS.fields_by_name['liquidator_reward_share_rate']._serialized_options = b'\332\336\037&github.com/cosmos/cosmos-sdk/types.Dec\310\336\037\000'
  _PARAMS.fields_by_name['binary_options_market_instant_listing_fee']._options = None
  _PARAMS.fields_by_name['binary_options_market_instant_listing_fee']._serialized_options = b'\310\336\037\000'
  _PARAMS.fields_by_name['spot_atomic_market_order_fee_multiplier']._options = None
  _PARAMS.fields_by_name['spot_atomic_market_order_fee_multiplier']._serialized_options = b'\332\336\037&github.com/cosmos/cosmos-sdk/types.Dec\310\336\037\000'
  _PARAMS.fields_by_name['derivative_atomic_market_order_fee_multiplier']._options = None
  _PARAMS.fields_by_name['derivative_atomic_market_order_fee_multiplier']._serialized_options = b'\332\336\037&github.com/cosmos/cosmos-sdk/types.Dec\310\336\037\000'
  _PARAMS.fields_by_name['binary_options_atomic_market_order_fee_multiplier']._options = None
  _PARAMS.fields_by_name['binary_options_atomic_market_order_fee_multiplier']._serialized_options = b'\332\336\037&github.com/cosmos/cosmos-sdk/types.Dec\310\336\037\000'
  _PARAMS.fields_by_name['minimal_protocol_fee_rate']._options = None
  _PARAMS.fields_by_name['minimal_protocol_fee_rate']._serialized_options = b'\332\336\037&github.com/cosmos/cosmos-sdk/types.Dec\310\336\037\000'
  _PARAMS._options = None
  _PARAMS._serialized_options = b'\350\240\037\001'
  _MARKETFEEMULTIPLIER.fields_by_name['fee_multiplier']._options = None
  _MARKETFEEMULTIPLIER.fields_by_name['fee_multiplier']._serialized_options = b'\332\336\037&github.com/cosmos/cosmos-sdk/types.Dec\310\336\037\000'
  _MARKETFEEMULTIPLIER._options = None
  _MARKETFEEMULTIPLIER._serialized_options = b'\210\240\037\000'
  _DERIVATIVEMARKET.fields_by_name['initial_margin_ratio']._options = None
  _DERIVATIVEMARKET.fields_by_name['initial_margin_ratio']._serialized_options = b'\332\336\037&github.com/cosmos/cosmos-sdk/types.Dec\310\336\037\000'
  _DERIVATIVEMARKET.fields_by_name['maintenance_margin_ratio']._options = None
  _DERIVATIVEMARKET.fields_by_name['maintenance_margin_ratio']._serialized_options = b'\332\336\037&github.com/cosmos/cosmos-sdk/types.Dec\310\336\037\000'
  _DERIVATIVEMARKET.fields_by_name['maker_fee_rate']._options = None
  _DERIVATIVEMARKET.fields_by_name['maker_fee_rate']._serialized_options = b'\332\336\037&github.com/cosmos/cosmos-sdk/types.Dec\310\336\037\000'
  _DERIVATIVEMARKET.fields_by_name['taker_fee_rate']._options = None
  _DERIVATIVEMARKET.fields_by_name['taker_fee_rate']._serialized_options = b'\332\336\037&github.com/cosmos/cosmos-sdk/types.Dec\310\336\037\000'
  _DERIVATIVEMARKET.fields_by_name['relayer_fee_share_rate']._options = None
  _DERIVATIVEMARKET.fields_by_name['relayer_fee_share_rate']._serialized_options = b'\332\336\037&github.com/cosmos/cosmos-sdk/types.Dec\310\336\037\000'
  _DERIVATIVEMARKET.fields_by_name['min_price_tick_size']._options = None
  _DERIVATIVEMARKET.fields_by_name['min_price_tick_size']._serialized_options = b'\332\336\037&github.com/cosmos/cosmos-sdk/types.Dec\310\336\037\000'
  _DERIVATIVEMARKET.fields_by_name['min_quantity_tick_size']._options = None
  _DERIVATIVEMARKET.fields_by_name['min_quantity_tick_size']._serialized_options = b'\332\336\037&github.com/cosmos/cosmos-sdk/types.Dec\310\336\037\000'
  _DERIVATIVEMARKET._options = None
  _DERIVATIVEMARKET._serialized_options = b'\210\240\037\000'
  _BINARYOPTIONSMARKET.fields_by_name['maker_fee_rate']._options = None
  _BINARYOPTIONSMARKET.fields_by_name['maker_fee_rate']._serialized_options = b'\332\336\037&github.com/cosmos/cosmos-sdk/types.Dec\310\336\037\000'
  _BINARYOPTIONSMARKET.fields_by_name['taker_fee_rate']._options = None
  _BINARYOPTIONSMARKET.fields_by_name['taker_fee_rate']._serialized_options = b'\332\336\037&github.com/cosmos/cosmos-sdk/types.Dec\310\336\037\000'
  _BINARYOPTIONSMARKET.fields_by_name['relayer_fee_share_rate']._options = None
  _BINARYOPTIONSMARKET.fields_by_name['relayer_fee_share_rate']._serialized_options = b'\332\336\037&github.com/cosmos/cosmos-sdk/types.Dec\310\336\037\000'
  _BINARYOPTIONSMARKET.fields_by_name['min_price_tick_size']._options = None
  _BINARYOPTIONSMARKET.fields_by_name['min_price_tick_size']._serialized_options = b'\332\336\037&github.com/cosmos/cosmos-sdk/types.Dec\310\336\037\000'
  _BINARYOPTIONSMARKET.fields_by_name['min_quantity_tick_size']._options = None
  _BINARYOPTIONSMARKET.fields_by_name['min_quantity_tick_size']._serialized_options = b'\332\336\037&github.com/cosmos/cosmos-sdk/types.Dec\310\336\037\000'
  _BINARYOPTIONSMARKET.fields_by_name['settlement_price']._options = None
  _BINARYOPTIONSMARKET.fields_by_name['settlement_price']._serialized_options = b'\332\336\037&github.com/cosmos/cosmos-sdk/types.Dec\310\336\037\001'
  _BINARYOPTIONSMARKET._options = None
  _BINARYOPTIONSMARKET._serialized_options = b'\210\240\037\000'
  _EXPIRYFUTURESMARKETINFO.fields_by_name['expiration_twap_start_price_cumulative']._options = None
  _EXPIRYFUTURESMARKETINFO.fields_by_name['expiration_twap_start_price_cumulative']._serialized_options = b'\332\336\037&github.com/cosmos/cosmos-sdk/types.Dec\310\336\037\000'
  _EXPIRYFUTURESMARKETINFO.fields_by_name['settlement_price']._options = None
  _EXPIRYFUTURESMARKETINFO.fields_by_name['settlement_price']._serialized_options = b'\332\336\037&github.com/cosmos/cosmos-sdk/types.Dec\310\336\037\000'
  _PERPETUALMARKETINFO.fields_by_name['hourly_funding_rate_cap']._options = None
  _PERPETUALMARKETINFO.fields_by_name['hourly_funding_rate_cap']._serialized_options = b'\332\336\037&github.com/cosmos/cosmos-sdk/types.Dec\310\336\037\000'
  _PERPETUALMARKETINFO.fields_by_name['hourly_interest_rate']._options = None
  _PERPETUALMARKETINFO.fields_by_name['hourly_interest_rate']._serialized_options = b'\332\336\037&github.com/cosmos/cosmos-sdk/types.Dec\310\336\037\000'
  _PERPETUALMARKETFUNDING.fields_by_name['cumulative_funding']._options = None
  _PERPETUALMARKETFUNDING.fields_by_name['cumulative_funding']._serialized_options = b'\332\336\037&github.com/cosmos/cosmos-sdk/types.Dec\310\336\037\000'
  _PERPETUALMARKETFUNDING.fields_by_name['cumulative_price']._options = None
  _PERPETUALMARKETFUNDING.fields_by_name['cumulative_price']._serialized_options = b'\332\336\037&github.com/cosmos/cosmos-sdk/types.Dec\310\336\037\000'
  _DERIVATIVEMARKETSETTLEMENTINFO.fields_by_name['settlement_price']._options = None
  _DERIVATIVEMARKETSETTLEMENTINFO.fields_by_name['settlement_price']._serialized_options = b'\332\336\037&github.com/cosmos/cosmos-sdk/types.Dec\310\336\037\000'
  _SPOTMARKET.fields_by_name['maker_fee_rate']._options = None
  _SPOTMARKET.fields_by_name['maker_fee_rate']._serialized_options = b'\332\336\037&github.com/cosmos/cosmos-sdk/types.Dec\310\336\037\000'
  _SPOTMARKET.fields_by_name['taker_fee_rate']._options = None
  _SPOTMARKET.fields_by_name['taker_fee_rate']._serialized_options = b'\332\336\037&github.com/cosmos/cosmos-sdk/types.Dec\310\336\037\000'
  _SPOTMARKET.fields_by_name['relayer_fee_share_rate']._options = None
  _SPOTMARKET.fields_by_name['relayer_fee_share_rate']._serialized_options = b'\332\336\037&github.com/cosmos/cosmos-sdk/types.Dec\310\336\037\000'
  _SPOTMARKET.fields_by_name['min_price_tick_size']._options = None
  _SPOTMARKET.fields_by_name['min_price_tick_size']._serialized_options = b'\332\336\037&github.com/cosmos/cosmos-sdk/types.Dec\310\336\037\000'
  _SPOTMARKET.fields_by_name['min_quantity_tick_size']._options = None
  _SPOTMARKET.fields_by_name['min_quantity_tick_size']._serialized_options = b'\332\336\037&github.com/cosmos/cosmos-sdk/types.Dec\310\336\037\000'
  _DEPOSIT.fields_by_name['available_balance']._options = None
  _DEPOSIT.fields_by_name['available_balance']._serialized_options = b'\332\336\037&github.com/cosmos/cosmos-sdk/types.Dec\310\336\037\000'
  _DEPOSIT.fields_by_name['total_balance']._options = None
  _DEPOSIT.fields_by_name['total_balance']._serialized_options = b'\332\336\037&github.com/cosmos/cosmos-sdk/types.Dec\310\336\037\000'
  _ORDERINFO.fields_by_name['price']._options = None
  _ORDERINFO.fields_by_name['price']._serialized_options = b'\332\336\037&github.com/cosmos/cosmos-sdk/types.Dec\310\336\037\000'
  _ORDERINFO.fields_by_name['quantity']._options = None
  _ORDERINFO.fields_by_name['quantity']._serialized_options = b'\332\336\037&github.com/cosmos/cosmos-sdk/types.Dec\310\336\037\000'
  _SPOTORDER.fields_by_name['order_info']._options = None
  _SPOTORDER.fields_by_name['order_info']._serialized_options = b'\310\336\037\000'
  _SPOTORDER.fields_by_name['trigger_price']._options = None
  _SPOTORDER.fields_by_name['trigger_price']._serialized_options = b'\332\336\037&github.com/cosmos/cosmos-sdk/types.Dec\310\336\037\001'
  _SPOTLIMITORDER.fields_by_name['order_info']._options = None
  _SPOTLIMITORDER.fields_by_name['order_info']._serialized_options = b'\310\336\037\000'
  _SPOTLIMITORDER.fields_by_name['fillable']._options = None
  _SPOTLIMITORDER.fields_by_name['fillable']._serialized_options = b'\332\336\037&github.com/cosmos/cosmos-sdk/types.Dec\310\336\037\000'
  _SPOTLIMITORDER.fields_by_name['trigger_price']._options = None
  _SPOTLIMITORDER.fields_by_name['trigger_price']._serialized_options = b'\332\336\037&github.com/cosmos/cosmos-sdk/types.Dec\310\336\037\001'
  _SPOTMARKETORDER.fields_by_name['order_info']._options = None
  _SPOTMARKETORDER.fields_by_name['order_info']._serialized_options = b'\310\336\037\000'
  _SPOTMARKETORDER.fields_by_name['balance_hold']._options = None
  _SPOTMARKETORDER.fields_by_name['balance_hold']._serialized_options = b'\332\336\037&github.com/cosmos/cosmos-sdk/types.Dec\310\336\037\000'
  _SPOTMARKETORDER.fields_by_name['trigger_price']._options = None
  _SPOTMARKETORDER.fields_by_name['trigger_price']._serialized_options = b'\332\336\037&github.com/cosmos/cosmos-sdk/types.Dec\310\336\037\001'
  _DERIVATIVEORDER.fields_by_name['order_info']._options = None
  _DERIVATIVEORDER.fields_by_name['order_info']._serialized_options = b'\310\336\037\000'
  _DERIVATIVEORDER.fields_by_name['margin']._options = None
  _DERIVATIVEORDER.fields_by_name['margin']._serialized_options = b'\332\336\037&github.com/cosmos/cosmos-sdk/types.Dec\310\336\037\000'
  _DERIVATIVEORDER.fields_by_name['trigger_price']._options = None
  _DERIVATIVEORDER.fields_by_name['trigger_price']._serialized_options = b'\332\336\037&github.com/cosmos/cosmos-sdk/types.Dec\310\336\037\001'
  _SUBACCOUNTORDERBOOKMETADATA.fields_by_name['aggregate_reduce_only_quantity']._options = None
  _SUBACCOUNTORDERBOOKMETADATA.fields_by_name['aggregate_reduce_only_quantity']._serialized_options = b'\332\336\037&github.com/cosmos/cosmos-sdk/types.Dec\310\336\037\000'
  _SUBACCOUNTORDERBOOKMETADATA.fields_by_name['aggregate_vanilla_quantity']._options = None
  _SUBACCOUNTORDERBOOKMETADATA.fields_by_name['aggregate_vanilla_quantity']._serialized_options = b'\332\336\037&github.com/cosmos/cosmos-sdk/types.Dec\310\336\037\000'
  _SUBACCOUNTORDER.fields_by_name['price']._options = None
  _SUBACCOUNTORDER.fields_by_name['price']._serialized_options = b'\332\336\037&github.com/cosmos/cosmos-sdk/types.Dec\310\336\037\000'
  _SUBACCOUNTORDER.fields_by_name['quantity']._options = None
  _SUBACCOUNTORDER.fields_by_name['quantity']._serialized_options = b'\332\336\037&github.com/cosmos/cosmos-sdk/types.Dec\310\336\037\000'
  _DERIVATIVELIMITORDER.fields_by_name['order_info']._options = None
  _DERIVATIVELIMITORDER.fields_by_name['order_info']._serialized_options = b'\310\336\037\000'
  _DERIVATIVELIMITORDER.fields_by_name['margin']._options = None
  _DERIVATIVELIMITORDER.fields_by_name['margin']._serialized_options = b'\332\336\037&github.com/cosmos/cosmos-sdk/types.Dec\310\336\037\000'
  _DERIVATIVELIMITORDER.fields_by_name['fillable']._options = None
  _DERIVATIVELIMITORDER.fields_by_name['fillable']._serialized_options = b'\332\336\037&github.com/cosmos/cosmos-sdk/types.Dec\310\336\037\000'
  _DERIVATIVELIMITORDER.fields_by_name['trigger_price']._options = None
  _DERIVATIVELIMITORDER.fields_by_name['trigger_price']._serialized_options = b'\332\336\037&github.com/cosmos/cosmos-sdk/types.Dec\310\336\037\001'
  _DERIVATIVEMARKETORDER.fields_by_name['order_info']._options = None
  _DERIVATIVEMARKETORDER.fields_by_name['order_info']._serialized_options = b'\310\336\037\000'
  _DERIVATIVEMARKETORDER.fields_by_name['margin']._options = None
  _DERIVATIVEMARKETORDER.fields_by_name['margin']._serialized_options = b'\332\336\037&github.com/cosmos/cosmos-sdk/types.Dec\310\336\037\000'
  _DERIVATIVEMARKETORDER.fields_by_name['margin_hold']._options = None
  _DERIVATIVEMARKETORDER.fields_by_name['margin_hold']._serialized_options = b'\332\336\037&github.com/cosmos/cosmos-sdk/types.Dec\310\336\037\000'
  _DERIVATIVEMARKETORDER.fields_by_name['trigger_price']._options = None
  _DERIVATIVEMARKETORDER.fields_by_name['trigger_price']._serialized_options = b'\332\336\037&github.com/cosmos/cosmos-sdk/types.Dec\310\336\037\001'
  _POSITION.fields_by_name['quantity']._options = None
  _POSITION.fields_by_name['quantity']._serialized_options = b'\332\336\037&github.com/cosmos/cosmos-sdk/types.Dec\310\336\037\000'
  _POSITION.fields_by_name['entry_price']._options = None
  _POSITION.fields_by_name['entry_price']._serialized_options = b'\332\336\037&github.com/cosmos/cosmos-sdk/types.Dec\310\336\037\000'
  _POSITION.fields_by_name['margin']._options = None
  _POSITION.fields_by_name['margin']._serialized_options = b'\332\336\037&github.com/cosmos/cosmos-sdk/types.Dec\310\336\037\000'
  _POSITION.fields_by_name['cumulative_funding_entry']._options = None
  _POSITION.fields_by_name['cumulative_funding_entry']._serialized_options = b'\332\336\037&github.com/cosmos/cosmos-sdk/types.Dec\310\336\037\000'
  _TRADELOG.fields_by_name['quantity']._options = None
  _TRADELOG.fields_by_name['quantity']._serialized_options = b'\332\336\037&github.com/cosmos/cosmos-sdk/types.Dec\310\336\037\000'
  _TRADELOG.fields_by_name['price']._options = None
  _TRADELOG.fields_by_name['price']._serialized_options = b'\332\336\037&github.com/cosmos/cosmos-sdk/types.Dec\310\336\037\000'
  _TRADELOG.fields_by_name['fee']._options = None
  _TRADELOG.fields_by_name['fee']._serialized_options = b'\332\336\037&github.com/cosmos/cosmos-sdk/types.Dec\310\336\037\000'
  _TRADELOG.fields_by_name['fee_recipient_address']._options = None
  _TRADELOG.fields_by_name['fee_recipient_address']._serialized_options = b'\310\336\037\001'
  _POSITIONDELTA.fields_by_name['execution_quantity']._options = None
  _POSITIONDELTA.fields_by_name['execution_quantity']._serialized_options = b'\332\336\037&github.com/cosmos/cosmos-sdk/types.Dec\310\336\037\000'
  _POSITIONDELTA.fields_by_name['execution_margin']._options = None
  _POSITIONDELTA.fields_by_name['execution_margin']._serialized_options = b'\332\336\037&github.com/cosmos/cosmos-sdk/types.Dec\310\336\037\000'
  _POSITIONDELTA.fields_by_name['execution_price']._options = None
  _POSITIONDELTA.fields_by_name['execution_price']._serialized_options = b'\332\336\037&github.com/cosmos/cosmos-sdk/types.Dec\310\336\037\000'
  _DERIVATIVETRADELOG.fields_by_name['payout']._options = None
  _DERIVATIVETRADELOG.fields_by_name['payout']._serialized_options = b'\332\336\037&github.com/cosmos/cosmos-sdk/types.Dec\310\336\037\000'
  _DERIVATIVETRADELOG.fields_by_name['fee']._options = None
  _DERIVATIVETRADELOG.fields_by_name['fee']._serialized_options = b'\332\336\037&github.com/cosmos/cosmos-sdk/types.Dec\310\336\037\000'
  _DERIVATIVETRADELOG.fields_by_name['fee_recipient_address']._options = None
  _DERIVATIVETRADELOG.fields_by_name['fee_recipient_address']._serialized_options = b'\310\336\037\001'
  _POINTSMULTIPLIER.fields_by_name['maker_points_multiplier']._options = None
  _POINTSMULTIPLIER.fields_by_name['maker_points_multiplier']._serialized_options = b'\332\336\037&github.com/cosmos/cosmos-sdk/types.Dec\310\336\037\000'
  _POINTSMULTIPLIER.fields_by_name['taker_points_multiplier']._options = None
  _POINTSMULTIPLIER.fields_by_name['taker_points_multiplier']._serialized_options = b'\332\336\037&github.com/cosmos/cosmos-sdk/types.Dec\310\336\037\000'
  _TRADINGREWARDCAMPAIGNBOOSTINFO.fields_by_name['spot_market_multipliers']._options = None
  _TRADINGREWARDCAMPAIGNBOOSTINFO.fields_by_name['spot_market_multipliers']._serialized_options = b'\310\336\037\000'
  _TRADINGREWARDCAMPAIGNBOOSTINFO.fields_by_name['derivative_market_multipliers']._options = None
  _TRADINGREWARDCAMPAIGNBOOSTINFO.fields_by_name['derivative_market_multipliers']._serialized_options = b'\310\336\037\000'
  _CAMPAIGNREWARDPOOL.fields_by_name['max_campaign_rewards']._options = None
  _CAMPAIGNREWARDPOOL.fields_by_name['max_campaign_rewards']._serialized_options = b'\310\336\037\000\252\337\037(github.com/cosmos/cosmos-sdk/types.Coins'
  _FEEDISCOUNTTIERINFO.fields_by_name['maker_discount_rate']._options = None
  _FEEDISCOUNTTIERINFO.fields_by_name['maker_discount_rate']._serialized_options = b'\332\336\037&github.com/cosmos/cosmos-sdk/types.Dec\310\336\037\000'
  _FEEDISCOUNTTIERINFO.fields_by_name['taker_discount_rate']._options = None
  _FEEDISCOUNTTIERINFO.fields_by_name['taker_discount_rate']._serialized_options = b'\332\336\037&github.com/cosmos/cosmos-sdk/types.Dec\310\336\037\000'
  _FEEDISCOUNTTIERINFO.fields_by_name['staked_amount']._options = None
  _FEEDISCOUNTTIERINFO.fields_by_name['staked_amount']._serialized_options = b'\332\336\037&github.com/cosmos/cosmos-sdk/types.Int\310\336\037\000'
  _FEEDISCOUNTTIERINFO.fields_by_name['volume']._options = None
  _FEEDISCOUNTTIERINFO.fields_by_name['volume']._serialized_options = b'\332\336\037&github.com/cosmos/cosmos-sdk/types.Dec\310\336\037\000'
  _VOLUMERECORD.fields_by_name['maker_volume']._options = None
  _VOLUMERECORD.fields_by_name['maker_volume']._serialized_options = b'\332\336\037&github.com/cosmos/cosmos-sdk/types.Dec\310\336\037\000'
  _VOLUMERECORD.fields_by_name['taker_volume']._options = None
  _VOLUMERECORD.fields_by_name['taker_volume']._serialized_options = b'\332\336\037&github.com/cosmos/cosmos-sdk/types.Dec\310\336\037\000'
  _ACCOUNTREWARDS.fields_by_name['rewards']._options = None
  _ACCOUNTREWARDS.fields_by_name['rewards']._serialized_options = b'\310\336\037\000\252\337\037(github.com/cosmos/cosmos-sdk/types.Coins'
  _TRADERECORD.fields_by_name['price']._options = None
  _TRADERECORD.fields_by_name['price']._serialized_options = b'\332\336\037&github.com/cosmos/cosmos-sdk/types.Dec\310\336\037\000'
  _TRADERECORD.fields_by_name['quantity']._options = None
  _TRADERECORD.fields_by_name['quantity']._serialized_options = b'\332\336\037&github.com/cosmos/cosmos-sdk/types.Dec\310\336\037\000'
  _LEVEL.fields_by_name['p']._options = None
  _LEVEL.fields_by_name['p']._serialized_options = b'\332\336\037&github.com/cosmos/cosmos-sdk/types.Dec\310\336\037\000'
  _LEVEL.fields_by_name['q']._options = None
  _LEVEL.fields_by_name['q']._serialized_options = b'\332\336\037&github.com/cosmos/cosmos-sdk/types.Dec\310\336\037\000'
  _MARKETVOLUME.fields_by_name['volume']._options = None
  _MARKETVOLUME.fields_by_name['volume']._serialized_options = b'\310\336\037\000'
  _ATOMICMARKETORDERACCESSLEVEL._serialized_start=12194
  _ATOMICMARKETORDERACCESSLEVEL._serialized_end=12310
  _MARKETSTATUS._serialized_start=12312
  _MARKETSTATUS._serialized_end=12396
  _ORDERTYPE._serialized_start=12399
  _ORDERTYPE._serialized_end=12714
  _EXECUTIONTYPE._serialized_start=12717
  _EXECUTIONTYPE._serialized_end=12892
  _ORDERMASK._serialized_start=12895
  _ORDERMASK._serialized_end=13160
  _PARAMS._serialized_start=167
  _PARAMS._serialized_end=2071
  _MARKETFEEMULTIPLIER._serialized_start=2073
  _MARKETFEEMULTIPLIER._serialized_end=2191
  _DERIVATIVEMARKET._serialized_start=2194
  _DERIVATIVEMARKET._serialized_end=3025
  _BINARYOPTIONSMARKET._serialized_start=3028
  _BINARYOPTIONSMARKET._serialized_end=3835
  _EXPIRYFUTURESMARKETINFO._serialized_start=3838
  _EXPIRYFUTURESMARKETINFO._serialized_end=4112
  _PERPETUALMARKETINFO._serialized_start=4115
  _PERPETUALMARKETINFO._serialized_end=4372
  _PERPETUALMARKETFUNDING._serialized_start=4375
  _PERPETUALMARKETFUNDING._serialized_end=4573
  _DERIVATIVEMARKETSETTLEMENTINFO._serialized_start=4575
  _DERIVATIVEMARKETSETTLEMENTINFO._serialized_end=4700
  _NEXTFUNDINGTIMESTAMP._serialized_start=4702
  _NEXTFUNDINGTIMESTAMP._serialized_end=4748
  _SPOTMARKET._serialized_start=4751
  _SPOTMARKET._serialized_end=5278
  _DEPOSIT._serialized_start=5281
  _DEPOSIT._serialized_end=5436
  _SUBACCOUNTTRADENONCE._serialized_start=5438
  _SUBACCOUNTTRADENONCE._serialized_end=5475
  _ORDERINFO._serialized_start=5478
  _ORDERINFO._serialized_end=5664
  _SPOTORDER._serialized_start=5667
  _SPOTORDER._serialized_end=5892
  _SPOTLIMITORDER._serialized_start=5895
  _SPOTLIMITORDER._serialized_end=6192
  _SPOTMARKETORDER._serialized_start=6195
  _SPOTMARKETORDER._serialized_end=6497
  _DERIVATIVEORDER._serialized_start=6500
  _DERIVATIVEORDER._serialized_end=6795
  _SUBACCOUNTORDERBOOKMETADATA._serialized_start=6798
  _SUBACCOUNTORDERBOOKMETADATA._serialized_end=7159
  _SUBACCOUNTORDER._serialized_start=7162
  _SUBACCOUNTORDER._serialized_end=7330
  _SUBACCOUNTORDERDATA._serialized_start=7332
  _SUBACCOUNTORDERDATA._serialized_end=7433
  _DERIVATIVELIMITORDER._serialized_start=7436
  _DERIVATIVELIMITORDER._serialized_end=7803
  _DERIVATIVEMARKETORDER._serialized_start=7806
  _DERIVATIVEMARKETORDER._serialized_end=8177
  _POSITION._serialized_start=8180
  _POSITION._serialized_end=8487
  _MARKETORDERINDICATOR._serialized_start=8489
  _MARKETORDERINDICATOR._serialized_end=8545
  _TRADELOG._serialized_start=8548
  _TRADELOG._serialized_end=8828
  _POSITIONDELTA._serialized_start=8831
  _POSITIONDELTA._serialized_end=9086
  _DERIVATIVETRADELOG._serialized_start=9089
  _DERIVATIVETRADELOG._serialized_end=9381
  _SUBACCOUNTPOSITION._serialized_start=9383
  _SUBACCOUNTPOSITION._serialized_end=9482
  _SUBACCOUNTDEPOSIT._serialized_start=9484
  _SUBACCOUNTDEPOSIT._serialized_end=9580
  _DEPOSITUPDATE._serialized_start=9582
  _DEPOSITUPDATE._serialized_end=9677
  _POINTSMULTIPLIER._serialized_start=9680
  _POINTSMULTIPLIER._serialized_end=9860
  _TRADINGREWARDCAMPAIGNBOOSTINFO._serialized_start=9863
  _TRADINGREWARDCAMPAIGNBOOSTINFO._serialized_end=10143
  _CAMPAIGNREWARDPOOL._serialized_start=10146
  _CAMPAIGNREWARDPOOL._serialized_end=10298
  _TRADINGREWARDCAMPAIGNINFO._serialized_start=10301
  _TRADINGREWARDCAMPAIGNINFO._serialized_end=10513
  _FEEDISCOUNTTIERINFO._serialized_start=10516
  _FEEDISCOUNTTIERINFO._serialized_end=10826
  _FEEDISCOUNTSCHEDULE._serialized_start=10829
  _FEEDISCOUNTSCHEDULE._serialized_end=11021
  _FEEDISCOUNTTIERTTL._serialized_start=11023
  _FEEDISCOUNTTIERTTL._serialized_end=11080
  _VOLUMERECORD._serialized_start=11083
  _VOLUMERECORD._serialized_end=11237
  _ACCOUNTREWARDS._serialized_start=11239
  _ACCOUNTREWARDS._serialized_end=11366
  _TRADERECORDS._serialized_start=11368
  _TRADERECORDS._serialized_end=11472
  _SUBACCOUNTIDS._serialized_start=11474
  _SUBACCOUNTIDS._serialized_end=11513
  _TRADERECORD._serialized_start=11516
  _TRADERECORD._serialized_end=11677
  _LEVEL._serialized_start=11679
  _LEVEL._serialized_end=11804
  _AGGREGATESUBACCOUNTVOLUMERECORD._serialized_start=11806
  _AGGREGATESUBACCOUNTVOLUMERECORD._serialized_end=11928
  _AGGREGATEACCOUNTVOLUMERECORD._serialized_start=11930
  _AGGREGATEACCOUNTVOLUMERECORD._serialized_end=12043
  _MARKETVOLUME._serialized_start=12045
  _MARKETVOLUME._serialized_end=12142
  _DENOMDECIMALS._serialized_start=12144
  _DENOMDECIMALS._serialized_end=12192
# @@protoc_insertion_point(module_scope)



import os
import time
import grpc
import datetime
import threading
from http.cookies import SimpleCookie
from typing import List, Optional, Tuple, Union

from .exceptions import NotFoundError, EmptyMsgError

from .proto.cosmos.base.abci.v1beta1 import abci_pb2 as abci_type

from .proto.cosmos.base.tendermint.v1beta1 import (
    query_pb2_grpc as tendermint_query_grpc,
    query_pb2 as tendermint_query,
)
from .proto.cosmos.auth.v1beta1 import (
    query_pb2_grpc as auth_query_grpc,
    query_pb2 as auth_query,
    auth_pb2 as auth_type,
)
from .proto.cosmos.authz.v1beta1 import (
    query_pb2_grpc as authz_query_grpc,
    query_pb2 as authz_query,
    authz_pb2 as authz_type,
)
from .proto.cosmos.tx.v1beta1 import (
    service_pb2_grpc as tx_service_grpc,
    service_pb2 as tx_service,
)
from .proto.exchange import (
    injective_accounts_rpc_pb2 as exchange_accounts_rpc_pb,
    injective_accounts_rpc_pb2_grpc as exchange_accounts_rpc_grpc,
    injective_oracle_rpc_pb2 as oracle_rpc_pb,
    injective_oracle_rpc_pb2_grpc as oracle_rpc_grpc,
    injective_insurance_rpc_pb2 as insurance_rpc_pb,
    injective_insurance_rpc_pb2_grpc as insurance_rpc_grpc,
    injective_spot_exchange_rpc_pb2 as spot_exchange_rpc_pb,
    injective_spot_exchange_rpc_pb2_grpc as spot_exchange_rpc_grpc,
    injective_derivative_exchange_rpc_pb2 as derivative_exchange_rpc_pb,
    injective_derivative_exchange_rpc_pb2_grpc as derivative_exchange_rpc_grpc,
    injective_meta_rpc_pb2 as exchange_meta_rpc_pb,
    injective_meta_rpc_pb2_grpc as exchange_meta_rpc_grpc,
    injective_explorer_rpc_pb2 as explorer_rpc_pb,
    injective_explorer_rpc_pb2_grpc as explorer_rpc_grpc,
    injective_auction_rpc_pb2 as auction_rpc_pb,
    injective_auction_rpc_pb2_grpc as auction_rpc_grpc,
)

from .constant import Network

DEFAULT_TIMEOUTHEIGHT_SYNC_INTERVAL = 10 # seconds
DEFAULT_TIMEOUTHEIGHT = 20 # blocks
DEFAULT_SESSION_RENEWAL_OFFSET = 120 # seconds
DEFAULT_BLOCK_TIME = 3 # seconds

class Client:
    def __init__(
        self,
        network: Network,
        insecure: bool = False,
        credentials = grpc.ssl_channel_credentials()
    ):

        # chain stubs
        self.chain_channel = (
            grpc.insecure_channel(network.grpc_endpoint)
            if insecure else grpc.secure_channel(network.grpc_endpoint, credentials)
        )
        self.stubCosmosTendermint = tendermint_query_grpc.ServiceStub(self.chain_channel)
        self.stubAuth = auth_query_grpc.QueryStub(self.chain_channel)
        self.stubAuthz = authz_query_grpc.QueryStub(self.chain_channel)
        self.stubTx = tx_service_grpc.ServiceStub(self.chain_channel)
        self.chain_cookie = ""
        self.exchange_cookie = ""
        self.timeout_height = 1

        # exchange stubs
        self.exchange_channel = (
            grpc.insecure_channel(network.grpc_exchange_endpoint)
            if insecure else grpc.secure_channel(network.grpc_exchange_endpoint, credentials)
        )

        self.stubMeta = exchange_meta_rpc_grpc.InjectiveMetaRPCStub(self.exchange_channel)
        self.stubExchangeAccount = exchange_accounts_rpc_grpc.InjectiveAccountsRPCStub(
            self.exchange_channel
        )
        self.stubOracle = oracle_rpc_grpc.InjectiveOracleRPCStub(self.exchange_channel)
        self.stubInsurance = insurance_rpc_grpc.InjectiveInsuranceRPCStub(
            self.exchange_channel
        )
        self.stubSpotExchange = spot_exchange_rpc_grpc.InjectiveSpotExchangeRPCStub(
            self.exchange_channel
        )
        self.stubDerivativeExchange = (
            derivative_exchange_rpc_grpc.InjectiveDerivativeExchangeRPCStub(
                self.exchange_channel
            )
        )
        self.stubExplorer = explorer_rpc_grpc.InjectiveExplorerRPCStub(self.exchange_channel)
        self.stubAuction = auction_rpc_grpc.InjectiveAuctionRPCStub(self.exchange_channel)

    def close_exchange_channel(self):
        self.exchange_channel.close()

    def close_chain_channel(self):
        self.chain_channel.close()

        # timeout height update routine
        self.sync_timeout_height()

    def sync_timeout_height(self):
        threading.Timer(DEFAULT_TIMEOUTHEIGHT_SYNC_INTERVAL, self.sync_timeout_height).start()
        block = self.get_latest_block()
        self.timeout_height = block.block.header.height + DEFAULT_TIMEOUTHEIGHT

    # cookie helper methods
    def get_cookie(self, type):
        metadata = None
        if type == "chain":
            if self.chain_cookie != "":
                metadata = (("cookie", self.chain_cookie),)

		        # format cookie date into RFC1123 standard
                cookie = SimpleCookie()
                cookie.load(self.chain_cookie)
                expires_at = cookie.get("grpc-cookie").get("expires")
                expires_at = expires_at.replace("-"," ")
                yyyy = "20{}".format(expires_at[12:14])
                expires_at = expires_at[:12] + yyyy + expires_at[14:]

                # parse expire field to unix timestamp
                expire_timestamp = datetime.datetime.strptime(expires_at, "%a, %d %b %Y %H:%M:%S GMT").timestamp()

                # renew session if timestamp diff < offset
                timestamp_diff = expire_timestamp - int(time.time())
                if timestamp_diff < DEFAULT_SESSION_RENEWAL_OFFSET:
                    # request and set new cookie
                    req = tendermint_query.GetLatestBlockRequest()
                    res, call = self.stubCosmosTendermint.GetLatestBlock.with_call(req)
                    self.set_cookie(call,type="chain")
                    time.sleep(DEFAULT_BLOCK_TIME)
                    metadata = (("cookie", self.chain_cookie),)
                    return metadata
            return metadata

        if type == "exchange":
            if self.exchange_cookie != "":
                metadata = (("cookie", self.exchange_cookie),)
        return metadata

    def set_cookie(self, call, type):
        new_cookie = None
        for k, v in call.initial_metadata():
            if k == "set-cookie":
                new_cookie = v

        if new_cookie == None:
            return

        if type == "chain":
            self.chain_cookie = new_cookie
        if type == "exchange":
            self.exchange_cookie = new_cookie

    # default client methods
    def get_latest_block(self) -> tendermint_query.GetLatestBlockResponse:
        return self.stubCosmosTendermint.GetLatestBlock(
            tendermint_query.GetLatestBlockRequest()
        )

    def get_account(self, address: str) -> Optional[auth_type.BaseAccount]:
        try:
            account_any = self.stubAuth.Account(
                auth_query.QueryAccountRequest(address=address)
            ).account
            account = auth_type.BaseAccount()
            if account_any.Is(account.DESCRIPTOR):
                account_any.Unpack(account)
                return account
        except:
            return None

    def get_request_id_by_tx_hash(self, tx_hash: bytes) -> List[int]:
        tx = self.stubTx.GetTx(tx_service.GetTxRequest(hash=tx_hash))
        request_ids = []
        for tx in tx.tx_response.logs:
            request_event = [
                event
                for event in tx.events
                if event.type == "request" or event.type == "report"
            ]
            if len(request_event) == 1:
                attrs = request_event[0].attributes
                attr_id = [attr for attr in attrs if attr.key == "id"]
                if len(attr_id) == 1:
                    request_id = attr_id[0].value
                    request_ids.append(int(request_id))
        if len(request_ids) == 0:
            raise NotFoundError("Request Id is not found")
        return request_ids

    def simulate_tx(
        self, tx_byte: bytes
    ) -> Tuple[Union[abci_type.SimulationResponse, grpc.RpcError], bool]:
        try:
            req = tx_service.SimulateRequest(tx_bytes=tx_byte)

            metadata = self.get_cookie(type="chain")
            res, call = self.stubTx.Simulate.with_call(req, metadata=metadata)
            self.set_cookie(call,type="chain")

            return res, True

        except grpc.RpcError as err:
            return err, False

    def send_tx_sync_mode(self, tx_byte: bytes) -> abci_type.TxResponse:
        req = tx_service.BroadcastTxRequest(
            tx_bytes=tx_byte, mode=tx_service.BroadcastMode.BROADCAST_MODE_SYNC
        )
        metadata = self.get_cookie(type="chain")
        res, call = self.stubTx.BroadcastTx.with_call(req, metadata=metadata)
        self.set_cookie(call,type="chain")

        return res.tx_response

    def send_tx_async_mode(self, tx_byte: bytes) -> abci_type.TxResponse:
        req = tx_service.BroadcastTxRequest(
            tx_bytes=tx_byte,
            mode=tx_service.BroadcastMode.BROADCAST_MODE_ASYNC
        )
        metadata = self.get_cookie(type="chain")
        res, call = self.stubTx.BroadcastTx.with_call(req, metadata=metadata)
        self.set_cookie(call,type="chain")

        return res.tx_response

    def send_tx_block_mode(self, tx_byte: bytes) -> abci_type.TxResponse:
        req = tx_service.BroadcastTxRequest(
            tx_bytes=tx_byte,
            mode=tx_service.BroadcastMode.BROADCAST_MODE_BLOCK
        )
        metadata = self.get_cookie(type="chain")
        res, call = self.stubTx.BroadcastTx.with_call(req, metadata=metadata)
        self.set_cookie(call,type="chain")

        return res.tx_response

    def get_chain_id(self) -> str:
        latest_block = self.get_latest_block()
        return latest_block.block.header.chain_id

    def get_grants(self, granter: str, grantee: str, **kwargs):
        return self.stubAuthz.Grants(
            authz_query.QueryGrantsRequest(
                granter=granter,
                grantee=grantee,
                msg_type_url=kwargs.get("msg_type_url"),
            )
        )

    # Injective Exchange client methods

    # Auction RPC

    def get_auction(self, bid_round: int):
        req = auction_rpc_pb.AuctionRequest(round=bid_round)
        return self.stubAuction.AuctionEndpoint(req)

    def get_auctions(self):
        req = auction_rpc_pb.AuctionsRequest()
        return self.stubAuction.Auctions(req)

    def stream_bids(self):
        req = auction_rpc_pb.StreamBidsRequest()
        return self.stubAuction.StreamBids(req)

    # Meta RPC

    def ping(self):
        req = exchange_meta_rpc_pb.PingRequest()
        return self.stubMeta.Ping(req)

    def version(self):
        req = exchange_meta_rpc_pb.VersionRequest()
        return self.stubMeta.Version(req)

    def info(self):
        req = exchange_meta_rpc_pb.InfoRequest(
            timestamp=int(time.time() * 1000),
        )
        return self.stubMeta.Info(req)

    def stream_keepalive(self):
        req = exchange_meta_rpc_pb.StreamKeepaliveRequest()
        return self.stubMeta.StreamKeepalive(req)

    # Explorer RPC

    def get_tx_by_hash(self, tx_hash: str):
        req = explorer_rpc_pb.GetTxByTxHashRequest(hash=tx_hash)
        return self.stubExplorer.GetTxByTxHash(req)

    # AccountsRPC

    def stream_subaccount_balance(self, subaccount_id: str):
        req = exchange_accounts_rpc_pb.StreamSubaccountBalanceRequest(
            subaccount_id=subaccount_id
        )
        return self.stubExchangeAccount.StreamSubaccountBalance(req)

    def get_subaccount_balance(self, subaccount_id: str, denom: str):
        req = exchange_accounts_rpc_pb.SubaccountBalanceRequest(
            subaccount_id=subaccount_id, denom=denom
        )
        return self.stubExchangeAccount.SubaccountBalanceEndpoint(req)

    def get_subaccount_list(self, account_address: str):
        req = exchange_accounts_rpc_pb.SubaccountsListRequest(
            account_address=account_address
        )
        return self.stubExchangeAccount.SubaccountsList(req)

    def get_subaccount_balances_list(self, subaccount_id: str):
        req = exchange_accounts_rpc_pb.SubaccountBalancesListRequest(
            subaccount_id=subaccount_id
        )
        return self.stubExchangeAccount.SubaccountBalancesList(req)

    def get_subaccount_history(self, subaccount_id: str, **kwargs):
        req = exchange_accounts_rpc_pb.SubaccountHistoryRequest(
            subaccount_id=subaccount_id,
            denom=kwargs.get("denom"),
            transfer_types=kwargs.get("transfer_types"),
        )
        return self.stubExchangeAccount.SubaccountHistory(req)

    def get_subaccount_order_summary(self, subaccount_id: str, **kwargs):
        req = exchange_accounts_rpc_pb.SubaccountOrderSummaryRequest(
            subaccount_id=subaccount_id,
            order_direction=kwargs.get("order_direction"),
            market_id=kwargs.get("market_id"),
        )
        return self.stubExchangeAccount.SubaccountOrderSummary(req)

    def get_order_states(self, **kwargs):
        req = exchange_accounts_rpc_pb.OrderStatesRequest(
            spot_order_hashes=kwargs.get("spot_order_hashes"),
            derivative_order_hashes=kwargs.get("derivative_order_hashes"),
        )
        return self.stubExchangeAccount.OrderStates(req)

    def get_portfolio(self, account_address: str):
        req = exchange_accounts_rpc_pb.PortfolioRequest(account_address=account_address)
        return self.stubExchangeAccount.Portfolio(req)

    def get_rewards(self, **kwargs):
        req = exchange_accounts_rpc_pb.RewardsRequest(
            account_address=kwargs.get("account_address"), epoch=kwargs.get("epoch")
        )
        return self.stubExchangeAccount.Rewards(req)

    # OracleRPC

    def stream_oracle_prices(
        self, base_symbol: str, quote_symbol: str, oracle_type: str
    ):
        req = oracle_rpc_pb.StreamPricesRequest(
            base_symbol=base_symbol, quote_symbol=quote_symbol, oracle_type=oracle_type
        )
        return self.stubOracle.StreamPrices(req)

    def get_oracle_prices(
        self,
        base_symbol: str,
        quote_symbol: str,
        oracle_type: str,
        oracle_scale_factor: int,
    ):
        req = oracle_rpc_pb.PriceRequest(
            base_symbol=base_symbol,
            quote_symbol=quote_symbol,
            oracle_type=oracle_type,
            oracle_scale_factor=oracle_scale_factor,
        )
        return self.stubOracle.Price(req)

    def get_oracle_list(self):
        req = oracle_rpc_pb.OracleListRequest()
        return self.stubOracle.OracleList(req)

    # InsuranceRPC

    def get_insurance_funds(self):
        req = insurance_rpc_pb.FundsRequest()
        return self.stubInsurance.Funds(req)

    def get_redemptions(self, **kwargs):
        req = insurance_rpc_pb.RedemptionsRequest(
            redeemer=kwargs.get("redeemer"),
            redemption_denom=kwargs.get("redemption_denom"),
            status=kwargs.get("status"),
        )
        return self.stubInsurance.Redemptions(req)

    # SpotRPC

    def get_spot_market(self, market_id: str):
        req = spot_exchange_rpc_pb.MarketRequest(market_id=market_id)
        return self.stubSpotExchange.Market(req)

    def get_spot_markets(self, **kwargs):
        req = spot_exchange_rpc_pb.MarketsRequest(
            market_status=kwargs.get("market_status"),
            base_denom=kwargs.get("base_denom"),
            quote_denom=kwargs.get("quote_denom"),
        )
        return self.stubSpotExchange.Markets(req)

    def stream_spot_markets(self, **kwargs):
        req = spot_exchange_rpc_pb.StreamMarketsRequest(
            market_ids=kwargs.get("market_ids"))
        metadata = self.get_cookie(type="exchange")
        res = self.stubSpotExchange.StreamMarkets(req, metadata=metadata)
        self.set_cookie(res,type="exchange")
        return res.tx_response

    def get_spot_orderbook(self, market_id: str):
        req = spot_exchange_rpc_pb.OrderbookRequest(market_id=market_id)
        return self.stubSpotExchange.Orderbook(req)

    def get_spot_orderbooks(self, market_ids: List):
        req = spot_exchange_rpc_pb.OrderbooksRequest(market_ids=market_ids)
        return self.stubSpotExchange.Orderbooks(req)

    def get_spot_orderbook_v2(self, market_id: str):
        req = spot_exchange_rpc_pb.OrderbookV2Request(market_id=market_id)
        return self.stubSpotExchange.OrderbookV2(req)

    def get_spot_orderbooks_v2(self, market_ids: List):
        req = spot_exchange_rpc_pb.OrderbooksV2Request(market_ids=market_ids)
        return self.stubSpotExchange.OrderbooksV2(req)

    def get_spot_orders(self, market_id: str, **kwargs):
        req = spot_exchange_rpc_pb.OrdersRequest(
            market_id=market_id,
            order_side=kwargs.get("order_side"),
            subaccount_id=kwargs.get("subaccount_id"),
        )
        return self.stubSpotExchange.Orders(req)

    def get_spot_trades(self, market_id: str, **kwargs):
        req = spot_exchange_rpc_pb.TradesRequest(
            market_id=market_id,
            execution_side=kwargs.get("execution_side"),
            direction=kwargs.get("direction"),
            subaccount_id=kwargs.get("subaccount_id"),
            skip=kwargs.get("skip"),
            limit=kwargs.get("limit"),
        )
        return self.stubSpotExchange.Trades(req)

    def stream_spot_orderbook(self, market_id: str):
        return self.stream_spot_orderbook_snapshot(market_ids=[market_id])

    def stream_spot_orderbook_snapshot(self, market_ids: List[str]):
        req = spot_exchange_rpc_pb.StreamOrderbookSnapshotRequest(market_ids=market_ids)
        metadata = self.get_cookie(type="exchange")
        res = self.stubSpotExchange.StreamOrderbookSnapshot(req, metadata=metadata)
        self.set_cookie(res, type="exchange")
        return res

    def stream_spot_orderbook_update(self, market_ids: List[str]):
        req = spot_exchange_rpc_pb.StreamOrderbookUpdateRequest(market_ids=market_ids)
        metadata = self.get_cookie(type="exchange")
        res = self.stubSpotExchange.StreamOrderbookUpdate(req, metadata=metadata)
        self.set_cookie(res, type="exchange")
        return res

    def stream_spot_orderbooks(self, market_ids: List):
        req = spot_exchange_rpc_pb.StreamOrderbookRequest(market_ids=market_ids)
        metadata = self.get_cookie(type="exchange")
        res = self.stubSpotExchange.StreamOrderbook(req, metadata=metadata)
        self.set_cookie(res,type="exchange")
        return res

    def stream_spot_orders(self, market_id: str, **kwargs):
        req = spot_exchange_rpc_pb.StreamOrdersRequest(
            market_id=market_id,
            order_side=kwargs.get("order_side"),
            subaccount_id=kwargs.get("subaccount_id"),
        )
        metadata = self.get_cookie(type="exchange")
        res = self.stubSpotExchange.StreamOrders(req, metadata=metadata)
        self.set_cookie(res,type="exchange")
        return res

    def stream_spot_trades(self, **kwargs):
        req = spot_exchange_rpc_pb.StreamTradesRequest(
            market_id=kwargs.get("market_id"),
            market_ids=kwargs.get("market_ids"),
            execution_side=kwargs.get("execution_side"),
            direction=kwargs.get("direction"),
            subaccount_id=kwargs.get("subaccount_id"),
            subaccount_ids=kwargs.get("subaccount_ids"),
            skip=kwargs.get("skip"),
            limit=kwargs.get("limit"),
        )
        metadata = self.get_cookie(type="exchange")
        res = self.stubSpotExchange.StreamTrades(req, metadata=metadata)
        self.set_cookie(res,type="exchange")
        return res

    def get_spot_subaccount_orders(self, subaccount_id: str, **kwargs):
        req = spot_exchange_rpc_pb.SubaccountOrdersListRequest(
            subaccount_id=subaccount_id, market_id=kwargs.get("market_id")
        )
        return self.stubSpotExchange.SubaccountOrdersList(req)

    def get_spot_subaccount_trades(self, subaccount_id: str, **kwargs):
        req = spot_exchange_rpc_pb.SubaccountTradesListRequest(
            subaccount_id=subaccount_id,
            market_id=kwargs.get("market_id"),
            execution_type=kwargs.get("execution_type"),
            direction=kwargs.get("direction"),
        )
        return self.stubSpotExchange.SubaccountTradesList(req)

    # DerivativeRPC

    def get_derivative_market(self, market_id: str):
        req = derivative_exchange_rpc_pb.MarketRequest(market_id=market_id)
        return self.stubDerivativeExchange.Market(req)

    def get_derivative_markets(self, **kwargs):
        req = derivative_exchange_rpc_pb.MarketsRequest(
            market_status=kwargs.get("market_status"),
            quote_denom=kwargs.get("quote_denom"),
        )
        return self.stubDerivativeExchange.Markets(req)

    def stream_derivative_markets(self, **kwargs):
        req = derivative_exchange_rpc_pb.StreamMarketRequest(
            market_ids=kwargs.get("market_ids"))
        metadata = self.get_cookie(type="exchange")
        res = self.stubDerivativeExchange.StreamMarket(req, metadata=metadata)
        self.set_cookie(res,type="exchange")
        return res

    def get_derivative_orderbook(self, market_id: str):
        req = derivative_exchange_rpc_pb.OrderbookRequest(market_id=market_id)
        return self.stubDerivativeExchange.Orderbook(req)

    def get_derivative_orderbooks(self, market_ids: List):
        req = derivative_exchange_rpc_pb.OrderbooksRequest(market_ids=market_ids)
        return self.stubDerivativeExchange.Orderbooks(req)

    def get_derivative_orderbook_v2(self, market_id: str):
        req = derivative_exchange_rpc_pb.OrderbookV2Request(market_id=market_id)
        return self.stubDerivativeExchange.OrderbookV2(req)

    def get_derivative_orderbooks_v2(self, market_ids: List):
        req = derivative_exchange_rpc_pb.OrderbooksV2Request(market_ids=market_ids)
        return self.stubDerivativeExchange.OrderbooksV2(req)

    def get_derivative_orders(self, market_id: str, **kwargs):
        req = derivative_exchange_rpc_pb.OrdersRequest(
            market_id=market_id,
            order_side=kwargs.get("order_side"),
            subaccount_id=kwargs.get("subaccount_id"),
        )
        return self.stubDerivativeExchange.Orders(req)

    def get_derivative_trades(self, market_id: str, **kwargs):
        req = derivative_exchange_rpc_pb.TradesRequest(
            market_id=market_id,
            subaccount_id=kwargs.get("subaccount_id"),
            execution_side=kwargs.get("execution_side"),
            direction=kwargs.get("direction"),
            skip=kwargs.get("skip"),
            limit=kwargs.get("limit"),
        )
        return self.stubDerivativeExchange.Trades(req)

    # deprecated: use stream_derivative_orderbook_snapshot
    def stream_derivative_orderbook(self, market_id: str):
        return self.stream_derivative_orderbook_snapshot(market_ids=[market_id])

    def stream_derivative_orderbook_snapshot(self, market_ids: List[str]):
        req = derivative_exchange_rpc_pb.StreamOrderbookSnapshotRequest(market_ids=market_ids)
        metadata = self.get_cookie(type="exchange")
        res = self.stubDerivativeExchange.StreamOrderbookSnapshot(req, metadata=metadata)
        self.set_cookie(res, type="exchange")
        return res

    def stream_derivative_orderbook_update(self, market_ids: List[str]):
        req = derivative_exchange_rpc_pb.StreamOrderbookUpdateRequest(market_ids=market_ids)
        metadata = self.get_cookie(type="exchange")
        res = self.stubDerivativeExchange.StreamOrderbookUpdate(req, metadata=metadata)
        self.set_cookie(res, type="exchange")
        return res

    def stream_derivative_orderbooks(self, market_ids: List):
        req = derivative_exchange_rpc_pb.StreamOrderbookRequest(market_ids=market_ids)
        metadata = self.get_cookie(type="exchange")
        res = self.stubDerivativeExchange.StreamOrderbook(req, metadata=metadata)
        self.set_cookie(res,type="exchange")
        return res

    def stream_derivative_orders(self, market_id: str, **kwargs):
        req = derivative_exchange_rpc_pb.StreamOrdersRequest(
            market_id=market_id,
            order_side=kwargs.get("order_side"),
            subaccount_id=kwargs.get("subaccount_id"),
        )
        metadata = self.get_cookie(type="exchange")
        res = self.stubDerivativeExchange.StreamOrders(req, metadata=metadata)
        self.set_cookie(res,type="exchange")
        return res

    def stream_derivative_trades(self, **kwargs):
        req = derivative_exchange_rpc_pb.StreamTradesRequest(
            market_id=kwargs.get("market_id"),
            market_ids=kwargs.get("market_ids"),
            subaccount_id=kwargs.get("subaccount_id"),
            subaccount_ids=kwargs.get("subaccount_ids"),
            execution_side=kwargs.get("execution_side"),
            direction=kwargs.get("direction"),
            skip=kwargs.get("skip"),
            limit=kwargs.get("limit"),
        )
        metadata = self.get_cookie(type="exchange")
        res = self.stubDerivativeExchange.StreamTrades(req, metadata=metadata)
        self.set_cookie(res,type="exchange")
        return res

    def get_derivative_positions(self, market_id: str, **kwargs):
        req = derivative_exchange_rpc_pb.PositionsRequest(
            market_id=market_id, subaccount_id=kwargs.get("subaccount_id")
        )
        return self.stubDerivativeExchange.Positions(req)

    def stream_derivative_positions(self, **kwargs):
        req = derivative_exchange_rpc_pb.StreamPositionsRequest(
            market_id=kwargs.get("market_id"),
            market_ids=kwargs.get("market_ids"),
            subaccount_id=kwargs.get("subaccount_id"),
            subaccount_ids=kwargs.get("subaccount_ids")
        )
        metadata = self.get_cookie(type="exchange")
        res = self.stubDerivativeExchange.StreamPositions(req, metadata=metadata)
        self.set_cookie(res,type="exchange")
        return res

    def get_derivative_liquidable_positions(self, **kwargs):
        req = derivative_exchange_rpc_pb.LiquidablePositionsRequest(
            market_id=kwargs.get("market_id")
        )
        return self.stubDerivativeExchange.LiquidablePositions(req)

    def get_derivative_subaccount_orders(self, subaccount_id: str, **kwargs):
        req = derivative_exchange_rpc_pb.SubaccountOrdersListRequest(
            subaccount_id=subaccount_id, market_id=kwargs.get("market_id")
        )
        return self.stubDerivativeExchange.SubaccountOrdersList(req)

    def get_derivative_subaccount_trades(self, subaccount_id: str, **kwargs):
        req = derivative_exchange_rpc_pb.SubaccountTradesListRequest(
            subaccount_id=subaccount_id,
            market_id=kwargs.get("market_id"),
            execution_type=kwargs.get("execution_type"),
            direction=kwargs.get("direction"),
        )
        return self.stubDerivativeExchange.SubaccountTradesList(req)

    def get_funding_payments(self, subaccount_id: str, **kwargs):
        req = derivative_exchange_rpc_pb.FundingPaymentsRequest(
            subaccount_id=subaccount_id,
            market_id=kwargs.get("market_id"),
            skip=kwargs.get("skip"),
            limit=kwargs.get("limit"),
        )
        return self.stubDerivativeExchange.FundingPayments(req)

    def get_funding_rates(self, market_id: str, **kwargs):
        req = derivative_exchange_rpc_pb.FundingRatesRequest(
            market_id=market_id,
            skip=kwargs.get("skip"),
            limit=kwargs.get("limit"),
        )
        return self.stubDerivativeExchange.FundingRates(req)



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: injective/wasmx/v1/wasmx.proto
"""Generated protocol buffer code."""
from google.protobuf.internal import builder as _builder
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from gogoproto import gogo_pb2 as gogoproto_dot_gogo__pb2


DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n\x1einjective/wasmx/v1/wasmx.proto\x12\x12injective.wasmx.v1\x1a\x14gogoproto/gogo.proto\"\x86\x01\n\x06Params\x12\x1c\n\x14is_execution_enabled\x18\x01 \x01(\x08\x12!\n\x19max_begin_block_total_gas\x18\x02 \x01(\x04\x12\x1e\n\x16max_contract_gas_limit\x18\x03 \x01(\x04\x12\x15\n\rmin_gas_price\x18\x04 \x01(\x04:\x04\xe8\xa0\x1f\x01\"\x8b\x01\n\x12RegisteredContract\x12\x11\n\tgas_limit\x18\x01 \x01(\x04\x12\x11\n\tgas_price\x18\x02 \x01(\x04\x12\x15\n\ris_executable\x18\x03 \x01(\x08\x12\x15\n\x07\x63ode_id\x18\x04 \x01(\x04\x42\x04\xc8\xde\x1f\x01\x12\x1b\n\radmin_address\x18\x05 \x01(\tB\x04\xc8\xde\x1f\x01:\x04\xe8\xa0\x1f\x01\x42MZKgithub.com/InjectiveLabs/injective-core/injective-chain/modules/wasmx/typesb\x06proto3')

_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, globals())
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'injective.wasmx.v1.wasmx_pb2', globals())
if _descriptor._USE_C_DESCRIPTORS == False:

  DESCRIPTOR._options = None
  DESCRIPTOR._serialized_options = b'ZKgithub.com/InjectiveLabs/injective-core/injective-chain/modules/wasmx/types'
  _PARAMS._options = None
  _PARAMS._serialized_options = b'\350\240\037\001'
  _REGISTEREDCONTRACT.fields_by_name['code_id']._options = None
  _REGISTEREDCONTRACT.fields_by_name['code_id']._serialized_options = b'\310\336\037\001'
  _REGISTEREDCONTRACT.fields_by_name['admin_address']._options = None
  _REGISTEREDCONTRACT.fields_by_name['admin_address']._serialized_options = b'\310\336\037\001'
  _REGISTEREDCONTRACT._options = None
  _REGISTEREDCONTRACT._serialized_options = b'\350\240\037\001'
  _PARAMS._serialized_start=77
  _PARAMS._serialized_end=211
  _REGISTEREDCONTRACT._serialized_start=214
  _REGISTEREDCONTRACT._serialized_end=353
# @@protoc_insertion_point(module_scope)



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: cosmos/base/store/v1beta1/commit_info.proto
"""Generated protocol buffer code."""
from google.protobuf.internal import builder as _builder
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from gogoproto import gogo_pb2 as gogoproto_dot_gogo__pb2


DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n+cosmos/base/store/v1beta1/commit_info.proto\x12\x19\x63osmos.base.store.v1beta1\x1a\x14gogoproto/gogo.proto\"^\n\nCommitInfo\x12\x0f\n\x07version\x18\x01 \x01(\x03\x12?\n\x0bstore_infos\x18\x02 \x03(\x0b\x32$.cosmos.base.store.v1beta1.StoreInfoB\x04\xc8\xde\x1f\x00\"W\n\tStoreInfo\x12\x0c\n\x04name\x18\x01 \x01(\t\x12<\n\tcommit_id\x18\x02 \x01(\x0b\x32#.cosmos.base.store.v1beta1.CommitIDB\x04\xc8\xde\x1f\x00\"/\n\x08\x43ommitID\x12\x0f\n\x07version\x18\x01 \x01(\x03\x12\x0c\n\x04hash\x18\x02 \x01(\x0c:\x04\x98\xa0\x1f\x00\x42*Z(github.com/cosmos/cosmos-sdk/store/typesb\x06proto3')

_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, globals())
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'cosmos.base.store.v1beta1.commit_info_pb2', globals())
if _descriptor._USE_C_DESCRIPTORS == False:

  DESCRIPTOR._options = None
  DESCRIPTOR._serialized_options = b'Z(github.com/cosmos/cosmos-sdk/store/types'
  _COMMITINFO.fields_by_name['store_infos']._options = None
  _COMMITINFO.fields_by_name['store_infos']._serialized_options = b'\310\336\037\000'
  _STOREINFO.fields_by_name['commit_id']._options = None
  _STOREINFO.fields_by_name['commit_id']._serialized_options = b'\310\336\037\000'
  _COMMITID._options = None
  _COMMITID._serialized_options = b'\230\240\037\000'
  _COMMITINFO._serialized_start=96
  _COMMITINFO._serialized_end=190
  _STOREINFO._serialized_start=192
  _STOREINFO._serialized_end=279
  _COMMITID._serialized_start=281
  _COMMITID._serialized_end=328
# @@protoc_insertion_point(module_scope)
